searchData={"items":[{"type":"behaviour","title":"Jido","doc":"自動 (Jido) - A foundational framework for building autonomous, distributed agent systems in Elixir.\n\nThis module provides the main interface for interacting with Jido components, including:\n- Managing and interacting with Agents through a high-level API\n- Listing and retrieving Actions, Sensors, and Domains\n- Filtering and paginating results\n- Generating unique slugs for components","ref":"Jido.html"},{"type":"behaviour","title":"Agent Interaction Examples - Jido","doc":"# Find and act on an agent\n    \"agent-id\"\n    |> Jido.get_agent_by_id()\n    |> Jido.act(:command, %{param: \"value\"})\n\n    # Act asynchronously\n    {:ok, agent} = Jido.get_agent_by_id(\"agent-id\")\n    Jido.act_async(agent, :command)\n\n    # Send management commands\n    {:ok, agent} = Jido.get_agent_by_id(\"agent-id\")\n    Jido.manage(agent, :pause)\n\n    # Subscribe to agent events\n    {:ok, topic} = Jido.get_agent_topic(\"agent-id\")\n    Phoenix.PubSub.subscribe(MyApp.PubSub, topic)","ref":"Jido.html#module-agent-interaction-examples"},{"type":"function","title":"Jido.clone_agent/3","doc":"Clones an existing agent with a new ID.","ref":"Jido.html#clone_agent/3"},{"type":"function","title":"Parameters - Jido.clone_agent/3","doc":"- `source_id`: ID of the agent to clone\n- `new_id`: ID for the new cloned agent\n- `opts`: Optional keyword list of options to override for the new agent","ref":"Jido.html#clone_agent/3-parameters"},{"type":"function","title":"Returns - Jido.clone_agent/3","doc":"- `{:ok, pid}` with the new agent's process ID\n- `{:error, reason}` if cloning fails","ref":"Jido.html#clone_agent/3-returns"},{"type":"function","title":"Examples - Jido.clone_agent/3","doc":"iex> {:ok, new_pid} = Jido.clone_agent(\"source-agent\", \"cloned-agent\")\n    {:ok, #PID<0.125.0>}","ref":"Jido.html#clone_agent/3-examples"},{"type":"function","title":"Jido.cmd/4","doc":"Sends a command to an agent.","ref":"Jido.html#cmd/4"},{"type":"function","title":"Parameters - Jido.cmd/4","doc":"- `agent`: Agent pid or return value from get_agent\n- `action`: The action to execute\n- `args`: Optional map of action arguments\n- `opts`: Optional keyword list of options","ref":"Jido.html#cmd/4-parameters"},{"type":"function","title":"Returns - Jido.cmd/4","doc":"Returns the result of command execution.","ref":"Jido.html#cmd/4-returns"},{"type":"function","title":"Examples - Jido.cmd/4","doc":"iex> {:ok, agent} = Jido.get_agent(\"my-agent\")\n    iex> Jido.cmd(agent, :generate_response, %{prompt: \"Hello\"})\n    {:ok, %{response: \"Hi there!\"}}","ref":"Jido.html#cmd/4-examples"},{"type":"type","title":"Jido.component_metadata/0","doc":"","ref":"Jido.html#t:component_metadata/0"},{"type":"callback","title":"Jido.config/0","doc":"","ref":"Jido.html#c:config/0"},{"type":"function","title":"Jido.ensure_started/1","doc":"Callback used by the generated `start_link/0` function.\nThis is where we actually call Jido.Supervisor.start_link.","ref":"Jido.html#ensure_started/1"},{"type":"function","title":"Jido.get_action_by_slug/1","doc":"","ref":"Jido.html#get_action_by_slug/1"},{"type":"function","title":"Jido.get_agent/2","doc":"Retrieves a running Agent by its ID.","ref":"Jido.html#get_agent/2"},{"type":"function","title":"Parameters - Jido.get_agent/2","doc":"- `id`: String or atom ID of the agent to retrieve\n- `opts`: Optional keyword list of options:\n  - `:registry`: Override the default agent registry","ref":"Jido.html#get_agent/2-parameters"},{"type":"function","title":"Returns - Jido.get_agent/2","doc":"- `{:ok, pid}` if agent is found and running\n- `{:error, :not_found}` if agent doesn't exist","ref":"Jido.html#get_agent/2-returns"},{"type":"function","title":"Examples - Jido.get_agent/2","doc":"iex> {:ok, agent} = Jido.get_agent(\"my-agent\")\n    {:ok, #PID<0.123.0>}\n\n    # Using a custom registry\n    iex> {:ok, agent} = Jido.get_agent(\"my-agent\", registry: MyApp.Registry)\n    {:ok, #PID<0.123.0>}","ref":"Jido.html#get_agent/2-examples"},{"type":"function","title":"Jido.get_agent!/2","doc":"Pipe-friendly version of get_agent that raises on errors.","ref":"Jido.html#get_agent!/2"},{"type":"function","title":"Parameters - Jido.get_agent!/2","doc":"- `id`: String or atom ID of the agent to retrieve\n- `opts`: Optional keyword list of options:\n  - `:registry`: Override the default agent registry","ref":"Jido.html#get_agent!/2-parameters"},{"type":"function","title":"Returns - Jido.get_agent!/2","doc":"- `pid` if agent is found\n- Raises `RuntimeError` if agent not found","ref":"Jido.html#get_agent!/2-returns"},{"type":"function","title":"Examples - Jido.get_agent!/2","doc":"iex> \"my-agent\" |> Jido.get_agent!() |> Jido.cmd(:command)\n    :ok","ref":"Jido.html#get_agent!/2-examples"},{"type":"function","title":"Jido.get_agent_by_slug/1","doc":"","ref":"Jido.html#get_agent_by_slug/1"},{"type":"function","title":"Jido.get_agent_state/1","doc":"Gets the current state of an agent.","ref":"Jido.html#get_agent_state/1"},{"type":"function","title":"Parameters - Jido.get_agent_state/1","doc":"- `agent_or_id`: Agent pid, ID, or return value from get_agent","ref":"Jido.html#get_agent_state/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_state/1","doc":"- `{:ok, state}` with the agent's current state\n- `{:error, reason}` if state couldn't be retrieved","ref":"Jido.html#get_agent_state/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_state/1","doc":"iex> {:ok, state} = Jido.get_agent_state(\"my-agent\")\n    {:ok, %Jido.Agent.Server.State{...}}","ref":"Jido.html#get_agent_state/1-examples"},{"type":"function","title":"Jido.get_agent_status/1","doc":"Gets the status of an agent.","ref":"Jido.html#get_agent_status/1"},{"type":"function","title":"Parameters - Jido.get_agent_status/1","doc":"- `agent_or_id`: Agent pid, ID, or return value from get_agent","ref":"Jido.html#get_agent_status/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_status/1","doc":"- `{:ok, status}` with the agent's status\n- `{:error, reason}` if status couldn't be retrieved","ref":"Jido.html#get_agent_status/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_status/1","doc":"iex> {:ok, status} = Jido.get_agent_status(\"my-agent\")\n    {:ok, :idle}","ref":"Jido.html#get_agent_status/1-examples"},{"type":"function","title":"Jido.get_agent_supervisor/1","doc":"Gets the supervisor for an agent.","ref":"Jido.html#get_agent_supervisor/1"},{"type":"function","title":"Parameters - Jido.get_agent_supervisor/1","doc":"- `agent_or_id`: Agent pid, ID, or return value from get_agent","ref":"Jido.html#get_agent_supervisor/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_supervisor/1","doc":"- `{:ok, supervisor_pid}` with the agent's supervisor pid\n- `{:error, reason}` if supervisor couldn't be retrieved","ref":"Jido.html#get_agent_supervisor/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_supervisor/1","doc":"iex> {:ok, supervisor} = Jido.get_agent_supervisor(\"my-agent\")\n    {:ok, #PID<0.124.0>}","ref":"Jido.html#get_agent_supervisor/1-examples"},{"type":"function","title":"Jido.get_agent_topic/1","doc":"Gets the PubSub topic for an agent.","ref":"Jido.html#get_agent_topic/1"},{"type":"function","title":"Parameters - Jido.get_agent_topic/1","doc":"- `agent_or_id`: Agent pid, ID, or return value from get_agent","ref":"Jido.html#get_agent_topic/1-parameters"},{"type":"function","title":"Returns - Jido.get_agent_topic/1","doc":"- `{:ok, topic}` with the agent's topic string\n- `{:error, reason}` if topic couldn't be retrieved","ref":"Jido.html#get_agent_topic/1-returns"},{"type":"function","title":"Examples - Jido.get_agent_topic/1","doc":"iex> {:ok, topic} = Jido.get_agent_topic(\"my-agent\")\n    {:ok, \"jido.agent.my-agent\"}\n\n    iex> {:ok, agent} = Jido.get_agent(\"my-agent\")\n    iex> {:ok, topic} = Jido.get_agent_topic(agent)\n    {:ok, \"jido.agent.my-agent\"}","ref":"Jido.html#get_agent_topic/1-examples"},{"type":"function","title":"Jido.get_sensor_by_slug/1","doc":"","ref":"Jido.html#get_sensor_by_slug/1"},{"type":"function","title":"Jido.list_actions/1","doc":"","ref":"Jido.html#list_actions/1"},{"type":"function","title":"Jido.list_agents/1","doc":"","ref":"Jido.html#list_agents/1"},{"type":"function","title":"Jido.list_sensors/1","doc":"","ref":"Jido.html#list_sensors/1"},{"type":"function","title":"Jido.prompt/1","doc":"Retrieves a prompt file from the priv/prompts directory by its name.","ref":"Jido.html#prompt/1"},{"type":"function","title":"Parameters - Jido.prompt/1","doc":"- `name`: An atom representing the name of the prompt file (without .txt extension)","ref":"Jido.html#prompt/1-parameters"},{"type":"function","title":"Returns - Jido.prompt/1","doc":"The contents of the prompt file as a string if found, otherwise raises an error.","ref":"Jido.html#prompt/1-returns"},{"type":"function","title":"Examples - Jido.prompt/1","doc":"iex> Jido.prompt(:system)\n    \"You are a helpful AI assistant...\"\n\n    iex> Jido.prompt(:nonexistent)\n    ** (File.Error) could not read file priv/prompts/nonexistent.txt","ref":"Jido.html#prompt/1-examples"},{"type":"behaviour","title":"Jido.Action","doc":"Defines a discrete, composable unit of functionality within the Jido system.\n\nEach Action represents a delayed computation that can be composed with others\nto build complex workflows and workflows. Actions are defined at compile-time\nand provide a consistent interface for validating inputs, executing workflows,\nand handling results.","ref":"Jido.Action.html"},{"type":"behaviour","title":"Features - Jido.Action","doc":"- Compile-time configuration validation\n- Runtime input parameter validation\n- Consistent error handling and formatting\n- Extensible lifecycle hooks\n- JSON serialization support","ref":"Jido.Action.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Action","doc":"To define a new Action, use the `Jido.Action` behavior in your module:\n\n    defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        # Your action logic here\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Action","doc":"Implementing modules must define the following callback:\n\n- `c:run/2`: Executes the main logic of the Action.\n\nOptional callbacks for custom behavior:\n\n- `c:on_before_validate_params/1`: Called before parameter validation.\n- `c:on_after_validate_params/1`: Called after parameter validation.\n- `c:on_after_run/1`: Called after the Action's main logic has executed.","ref":"Jido.Action.html#module-callbacks"},{"type":"behaviour","title":"Error Handling - Jido.Action","doc":"Actions use the `OK` monad for consistent error handling. Errors are wrapped\nin `Jido.Error` structs for uniform error reporting across the system.","ref":"Jido.Action.html#module-error-handling"},{"type":"behaviour","title":"Parameter Validation - Jido.Action","doc":"> **Note on Validation:** The validation process for Actions is intentionally open.\n> Only fields specified in the schema are validated. Unspecified fields are not\n> validated, allowing for easier Action composition. This approach enables Actions\n> to accept and pass along additional parameters that may be required by other\n> Actions in a chain without causing validation errors.","ref":"Jido.Action.html#module-parameter-validation"},{"type":"macro","title":"Jido.Action.__using__/1","doc":"Defines a new Action module.\n\nThis macro sets up the necessary structure and callbacks for a Action,\nincluding configuration validation and default implementations.","ref":"Jido.Action.html#__using__/1"},{"type":"macro","title":"Options - Jido.Action.__using__/1","doc":"* `:name` - Required. The name of the Action. Must contain only letters, numbers, and underscores.\n\n* `:description` (`t:String.t/0`) - A description of what the Action does.\n\n* `:category` (`t:String.t/0`) - The category of the Action.\n\n* `:tags` (list of `t:String.t/0`) - A list of tags associated with the Action. The default value is `[]`.\n\n* `:vsn` (`t:String.t/0`) - The version of the Action.\n\n* `:compensation` (`t:keyword/0`) - The default value is `[]`.\n\n  * `:enabled` (`t:boolean/0`) - The default value is `false`.\n\n  * `:max_retries` (`t:non_neg_integer/0`) - The default value is `1`.\n\n  * `:timeout` (`t:non_neg_integer/0`) - The default value is `5000`.\n\n* `:schema` (`t:keyword/0`) - A NimbleOptions schema for validating the Action's input parameters. The default value is `[]`.","ref":"Jido.Action.html#__using__/1-options"},{"type":"macro","title":"Examples - Jido.Action.__using__/1","doc":"defmodule MyAction do\n      use Jido.Action,\n        name: \"my_action\",\n        description: \"Performs a specific workflow\",\n        schema: [\n          input: [type: :string, required: true]\n        ]\n\n      @impl true\n      def run(params, _context) do\n        {:ok, %{result: String.upcase(params.input)}}\n      end\n    end","ref":"Jido.Action.html#__using__/1-examples"},{"type":"function","title":"Jido.Action.new/0","doc":"Raises an error indicating that Actions cannot be defined at runtime.\n\nThis function exists to prevent misuse of the Action system, as Actions\nare designed to be defined at compile-time only.","ref":"Jido.Action.html#new/0"},{"type":"function","title":"Returns - Jido.Action.new/0","doc":"Always returns `{:error, reason}` where `reason` is a config error.","ref":"Jido.Action.html#new/0-returns"},{"type":"function","title":"Examples - Jido.Action.new/0","doc":"iex> Jido.Action.new()\n    {:error, %Jido.Error{type: :config_error, message: \"Actions should not be defined at runtime\"}}","ref":"Jido.Action.html#new/0-examples"},{"type":"function","title":"Jido.Action.new/1","doc":"","ref":"Jido.Action.html#new/1"},{"type":"callback","title":"Jido.Action.on_after_run/1","doc":"Called after the Action's main logic has executed.\n\nThis optional callback allows for post-processing of the Action's result\nbefore it is returned to the caller.","ref":"Jido.Action.html#c:on_after_run/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_run/1","doc":"- `result`: The result map returned by the `run/2` function.","ref":"Jido.Action.html#c:on_after_run/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_run/1","doc":"- `{:ok, modified_result}` where `modified_result` is a potentially modified result map.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_run/1-returns"},{"type":"callback","title":"Jido.Action.on_after_validate_params/1","doc":"Called after parameter validation.\n\nThis optional callback allows for post-processing of validated parameters\nbefore they are passed to the `run/2` function.","ref":"Jido.Action.html#c:on_after_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_after_validate_params/1","doc":"- `params`: A map of validated input parameters.","ref":"Jido.Action.html#c:on_after_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_after_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if post-processing fails.","ref":"Jido.Action.html#c:on_after_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_before_validate_params/1","doc":"Called before parameter validation.\n\nThis optional callback allows for pre-processing of input parameters\nbefore they are validated against the Action's schema.","ref":"Jido.Action.html#c:on_before_validate_params/1"},{"type":"callback","title":"Parameters - Jido.Action.on_before_validate_params/1","doc":"- `params`: A map of raw input parameters.","ref":"Jido.Action.html#c:on_before_validate_params/1-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_before_validate_params/1","doc":"- `{:ok, modified_params}` where `modified_params` is a map of potentially modified parameters.\n- `{:error, reason}` if pre-processing fails.","ref":"Jido.Action.html#c:on_before_validate_params/1-returns"},{"type":"callback","title":"Jido.Action.on_error/4","doc":"Handles errors and performs compensation when enabled.\n\nCalled when an error occurs during Action execution if compensation is enabled\nin the Action's configuration.","ref":"Jido.Action.html#c:on_error/4"},{"type":"callback","title":"Parameters - Jido.Action.on_error/4","doc":"- `failed_params`: The parameters that were passed to the failed execution\n- `error`: The Error struct describing what went wrong\n- `context`: The execution context at the time of failure\n- `opts`: Additional options for compensation handling","ref":"Jido.Action.html#c:on_error/4-parameters"},{"type":"callback","title":"Returns - Jido.Action.on_error/4","doc":"- `{:ok, result}` if compensation succeeded\n- `{:error, reason}` if compensation failed","ref":"Jido.Action.html#c:on_error/4-returns"},{"type":"callback","title":"Examples - Jido.Action.on_error/4","doc":"def on_error(params, error, context, opts) do\n      # Perform compensation logic\n      case rollback_changes(params) do\n        :ok -> {:ok, %{compensated: true, original_error: error}}\n        {:error, reason} -> {:error, \"Compensation failed: #{reason}\"}\n      end\n    end","ref":"Jido.Action.html#c:on_error/4-examples"},{"type":"callback","title":"Jido.Action.run/2","doc":"Executes the Action with the given parameters and context.\n\nThis callback must be implemented by modules using `Jido.Action`.","ref":"Jido.Action.html#c:run/2"},{"type":"callback","title":"Parameters - Jido.Action.run/2","doc":"- `params`: A map of validated input parameters.\n- `context`: A map containing any additional context for the workflow.","ref":"Jido.Action.html#c:run/2-parameters"},{"type":"callback","title":"Returns - Jido.Action.run/2","doc":"- `{:ok, result}` where `result` is a map containing the workflow's output.\n- `{:error, reason}` where `reason` describes why the workflow failed.","ref":"Jido.Action.html#c:run/2-returns"},{"type":"type","title":"Jido.Action.t/0","doc":"","ref":"Jido.Action.html#t:t/0"},{"type":"behaviour","title":"Jido.Agent","doc":"Defines an Agent within the Jido system - a compile-time defined entity for managing complex workflows\nthrough a sequence of Actions.","ref":"Jido.Agent.html"},{"type":"behaviour","title":"Overview - Jido.Agent","doc":"An Agent represents a stateful workflow executor that can plan and execute a series of Actions in a\ntype-safe and composable way. Agents provide a consistent interface for orchestrating complex operations\nwhile maintaining state validation, error handling, and extensibility through lifecycle hooks.","ref":"Jido.Agent.html#module-overview"},{"type":"behaviour","title":"Architecture - Jido.Agent","doc":"","ref":"Jido.Agent.html#module-architecture"},{"type":"behaviour","title":"Key Concepts - Jido.Agent","doc":"* Agents are defined at compile-time using the `use Jido.Agent` macro\n* Each Agent instance is created at server with a guaranteed unique ID\n* Agents maintain their own state schema for validation\n* Actions are registered with Agents and executed through a Runner\n* Lifecycle hooks enable customization of validation, planning and execution flows\n* All operations follow a consistent pattern returning `{:ok, result} | {:error, reason}`","ref":"Jido.Agent.html#module-key-concepts"},{"type":"behaviour","title":"Type Safety & Validation - Jido.Agent","doc":"* Configuration validated at compile-time via NimbleOptions\n* Server state changes validated against defined schema\n* Action modules checked for behavior implementation\n* Consistent return type enforcement across operations","ref":"Jido.Agent.html#module-type-safety-validation"},{"type":"behaviour","title":"Features - Jido.Agent","doc":"* Compile-time configuration validation\n* Server parameter validation via NimbleOptions\n* Comprehensive error handling with recovery hooks\n* Extensible lifecycle callbacks for all operations\n* JSON serialization support for persistence\n* Dynamic Action planning and execution\n* State management with validation and dirty tracking","ref":"Jido.Agent.html#module-features"},{"type":"behaviour","title":"Usage - Jido.Agent","doc":"","ref":"Jido.Agent.html#module-usage"},{"type":"behaviour","title":"Basic Example - Jido.Agent","doc":"defmodule MyAgent do\n      use Jido.Agent,\n        name: \"my_agent\",\n        description: \"Performs a complex workflow\",\n        category: \"processing\",\n        tags: [\"example\", \"demo\"],\n        vsn: \"1.0.0\",\n        schema: [\n          input: [type: :string, required: true],\n          status: [type: :atom, values: [:pending, :running, :complete]]\n        ],\n        actions: [MyAction1, MyAction2]\n    end\n\n    # Create and configure agent\n    {:ok, agent} = MyAgent.new()\n    {:ok, agent} = MyAgent.set(agent, input: \"test data\")\n\n    # Plan and execute actions\n    {:ok, agent} = MyAgent.plan(agent, MyAction1, %{value: 1})\n    {:ok, agent} = MyAgent.run(agent)  # Result stored in agent.result","ref":"Jido.Agent.html#module-basic-example"},{"type":"behaviour","title":"Customizing Behavior - Jido.Agent","doc":"defmodule CustomAgent do\n      use Jido.Agent,\n        name: \"custom_agent\",\n        schema: [status: [type: :atom]]\n\n      # Add pre-execution validation\n      def on_before_run(agent) do\n        if agent.state.status == :ready do\n          {:ok, agent}\n        else\n          {:error, \"Agent not ready\"}\n        end\n      end\n\n      # Custom error recovery\n      def on_error(agent, error) do\n        Logger.warning(\"Agent error\", error: error)\n        {:ok, %{agent | state: %{status: :error}}}\n      end\n    end","ref":"Jido.Agent.html#module-customizing-behavior"},{"type":"behaviour","title":"Callbacks - Jido.Agent","doc":"The following optional callbacks can be implemented to customize agent behavior.\nAll callbacks receive the agent struct and return `{:ok, agent} | {:error, reason}`.\n\n* `on_before_validate_state/1` - Pre-validation processing\n* `on_after_validate_state/1`  - Post-validation processing\n* `on_before_plan/3`           - Pre-planning processing with params\n* `on_before_run/1`            - Pre-execution validation/setup\n* `on_after_run/2`             - Post-execution processing\n* `on_after_directives/2`      - Post-directive application\n* `on_error/2`                 - Error handling and recovery\n\nDefault implementations pass through the agent unchanged.","ref":"Jido.Agent.html#module-callbacks"},{"type":"behaviour","title":"Important Notes - Jido.Agent","doc":"Each Agent module defines its own struct type and behavior. Agent functions must be called\non matching agent structs:\n\n  # Correct usage:\n  agent = MyAgent.new()\n  {:ok, agent} = MyAgent.set(agent, attrs)\n\n  # Incorrect usage:\n  agent = MyAgent.new()\n  {:ok, agent} = OtherAgent.set(agent, attrs)  # Will fail - type mismatch","ref":"Jido.Agent.html#module-important-notes"},{"type":"behaviour","title":"Runner Architecture - Jido.Agent","doc":"The Runner executes actions in the agent's pending_instructions queue.\n\n* Default implementation: `Jido.Runner.Simple`\n* Custom runners must implement the `Jido.Runner` behavior\n* Runners handle instruction execution and state management\n* Support for different execution strategies (simple, chain, parallel)","ref":"Jido.Agent.html#module-runner-architecture"},{"type":"behaviour","title":"Error Handling - Jido.Agent","doc":"Errors are returned as tagged tuples: `{:error, reason}`\n\nCommon error types:\n* `:validation_error` - Schema/parameter validation failures\n* `:execution_error`  - Action execution failures\n* `:directive_error`  - Directive application failures\n\nSee `Jido.Action` for implementing compatible Actions.","ref":"Jido.Agent.html#module-error-handling"},{"type":"behaviour","title":"Type Specifications - Jido.Agent","doc":"* `t()` - The Agent struct type\n* `instruction()` - Single action with params\n* `instructions()` - List of instructions\n* `agent_result()` - `{:ok, t()} | {:error, term()}`","ref":"Jido.Agent.html#module-type-specifications"},{"type":"type","title":"Jido.Agent.agent_result/0","doc":"","ref":"Jido.Agent.html#t:agent_result/0"},{"type":"function","title":"Jido.Agent.deregister_action/2","doc":"","ref":"Jido.Agent.html#deregister_action/2"},{"type":"type","title":"Jido.Agent.instruction/0","doc":"","ref":"Jido.Agent.html#t:instruction/0"},{"type":"type","title":"Jido.Agent.instructions/0","doc":"","ref":"Jido.Agent.html#t:instructions/0"},{"type":"type","title":"Jido.Agent.map_result/0","doc":"","ref":"Jido.Agent.html#t:map_result/0"},{"type":"function","title":"Jido.Agent.new/0","doc":"Raises an error indicating that Agents cannot be defined at server.\n\nThis function exists to prevent misuse of the Agent system, as Agents\nare designed to be defined at compile-time only.","ref":"Jido.Agent.html#new/0"},{"type":"function","title":"Returns - Jido.Agent.new/0","doc":"Always returns `{:error, reason}` where `reason` is a config error.","ref":"Jido.Agent.html#new/0-returns"},{"type":"function","title":"Examples - Jido.Agent.new/0","doc":"iex> Jido.Agent.new()\n    {:error, %Jido.Error{type: :config_error, message: \"Agents should not be defined at server\"}}","ref":"Jido.Agent.html#new/0-examples"},{"type":"function","title":"Jido.Agent.new/1","doc":"","ref":"Jido.Agent.html#new/1"},{"type":"callback","title":"Jido.Agent.on_after_directives/2","doc":"Called after successful directive application.\nAllows post-processing of directive results.","ref":"Jido.Agent.html#c:on_after_directives/2"},{"type":"callback","title":"Jido.Agent.on_after_run/2","doc":"Called after successful action execution.\nAllows post-processing of execution results.","ref":"Jido.Agent.html#c:on_after_run/2"},{"type":"callback","title":"Jido.Agent.on_after_validate_state/1","doc":"Called after state validation but before saving changes.\nAllows post-processing of validated state.","ref":"Jido.Agent.html#c:on_after_validate_state/1"},{"type":"callback","title":"Jido.Agent.on_before_plan/3","doc":"Called before planning actions, allows preprocessing of action parameters\nand potential action routing/transformation.","ref":"Jido.Agent.html#c:on_before_plan/3"},{"type":"callback","title":"Jido.Agent.on_before_run/1","doc":"Called after action planning but before execution.\nAllows inspection/modification of planned actions.","ref":"Jido.Agent.html#c:on_before_run/1"},{"type":"callback","title":"Jido.Agent.on_before_validate_state/1","doc":"Called before validating any state changes to the Agent.\nAllows custom preprocessing of state attributes.","ref":"Jido.Agent.html#c:on_before_validate_state/1"},{"type":"callback","title":"Jido.Agent.on_error/2","doc":"Called when any error occurs during the agent lifecycle.\nProvides error handling and recovery strategies.","ref":"Jido.Agent.html#c:on_error/2"},{"type":"function","title":"Jido.Agent.register_action/2","doc":"Registers one or more action modules with the agent at server. Registered actions\ncan be used in planning and execution. Action modules must implement the `Jido.Action`\nbehavior and pass validation checks.","ref":"Jido.Agent.html#register_action/2"},{"type":"function","title":"Action Requirements - Jido.Agent.register_action/2","doc":"* Must be valid Elixir modules implementing `Jido.Action` behavior\n* Must be loaded and available at server\n* Must pass validation via `Jido.Util.validate_actions/1`","ref":"Jido.Agent.html#register_action/2-action-requirements"},{"type":"function","title":"Parameters - Jido.Agent.register_action/2","doc":"* `agent` - The agent struct to update\n* `action_module` - Single action module or list of action modules to register","ref":"Jido.Agent.html#register_action/2-parameters"},{"type":"function","title":"Returns - Jido.Agent.register_action/2","doc":"* `{:ok, updated_agent}` - Agent struct with newly registered actions prepended\n* `{:error, String.t()}` - If action validation fails","ref":"Jido.Agent.html#register_action/2-returns"},{"type":"function","title":"Examples - Jido.Agent.register_action/2","doc":"# Register single action\n    {:ok, agent} = MyAgent.register_action(agent, MyApp.Actions.ProcessFile)\n\n    # Register multiple actions\n    {:ok, agent} = MyAgent.register_action(agent, [Action1, Action2])","ref":"Jido.Agent.html#register_action/2-examples"},{"type":"function","title":"Server Considerations - Jido.Agent.register_action/2","doc":"* Actions persist only for the agent's lifecycle\n* Duplicates are prevented during validation\n* Most recently registered actions take precedence\n\nSee `Jido.Action` for implementing actions and `Jido.Util.validate_actions/1` for validation details.","ref":"Jido.Agent.html#register_action/2-server-considerations"},{"type":"function","title":"Jido.Agent.registered_actions/1","doc":"Returns all action modules registered with the agent in registration order.\nIncludes both compile-time and server-registered actions.","ref":"Jido.Agent.html#registered_actions/1"},{"type":"function","title":"Parameters - Jido.Agent.registered_actions/1","doc":"* `agent` - The agent struct to inspect","ref":"Jido.Agent.html#registered_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.registered_actions/1","doc":"* `[module()]` - Action modules ordered by registration time (newest first)","ref":"Jido.Agent.html#registered_actions/1-returns"},{"type":"function","title":"Examples - Jido.Agent.registered_actions/1","doc":"agent = MyAgent.new()\n    MyAgent.registered_actions(agent) #=> [DefaultAction, CoreAction]\n\n    {:ok, agent} = MyAgent.register_action(agent, CustomAction)\n    MyAgent.registered_actions(agent) #=> [CustomAction, DefaultAction, CoreAction]\n\nSee `register_action/2` for adding actions and `plan/3` for using them.","ref":"Jido.Agent.html#registered_actions/1-examples"},{"type":"type","title":"Jido.Agent.t/0","doc":"","ref":"Jido.Agent.html#t:t/0"},{"type":"module","title":"Jido.Agent.Server","doc":"> This module is under active development and may undergo changes.\n\nA fault-tolerant, distributed agent server implementation that manages stateful agents\nwith built-in pub/sub capabilities and dynamic supervision.\n\nThe `Jido.Agent.Server` provides a robust framework for managing long-running agent processes\nwith the following key features:\n\n  * Automatic pub/sub event distribution via configurable PubSub adapter\n  * Dynamic supervision of child processes\n  * Configurable queue size limits and backpressure handling\n  * Graceful shutdown and cleanup of resources\n  * Comprehensive state management and transition handling\n  * Registry-based process naming for distributed operations","ref":"Jido.Agent.Server.html"},{"type":"module","title":"Usage - Jido.Agent.Server","doc":"To start an agent server:\n\n    {:ok, pid} = Jido.Agent.Server.start_link([\n      agent: MyAgent.new(),\n      pubsub: MyApp.PubSub,\n      name: \"my_agent\",\n      max_queue_size: 5_000\n    ])\n\nThe server can then receive commands via the `cmd/4` function:\n\n    Jido.Agent.Server.cmd(pid, :some_action, %{arg: \"value\"}, [])","ref":"Jido.Agent.Server.html#module-usage"},{"type":"module","title":"Configuration Options - Jido.Agent.Server","doc":"* `:agent` - Required. The agent struct or module implementing `new/0`\n  * `:pubsub` - Required. The PubSub module for event distribution\n  * `:name` - Optional. The registration name (defaults to agent.id)\n  * `:topic` - Optional. The PubSub topic (defaults to generated from agent.id)\n  * `:max_queue_size` - Optional. Maximum pending signals (defaults to 10,000)\n  * `:registry` - Optional. The Registry for process registration (defaults to Jido.AgentRegistry)","ref":"Jido.Agent.Server.html#module-configuration-options"},{"type":"module","title":"Supervision - Jido.Agent.Server","doc":"The server automatically supervises child processes using a `:one_for_one` DynamicSupervisor\nstrategy. Each child process is monitored and can be restarted independently.","ref":"Jido.Agent.Server.html#module-supervision"},{"type":"module","title":"State Management - Jido.Agent.Server","doc":"The server maintains its state through the `Jido.Agent.Server.State` struct and handles\ntransitions via signal processing. State transitions are validated and published as events.","ref":"Jido.Agent.Server.html#module-state-management"},{"type":"module","title":"Signal Processing - Jido.Agent.Server","doc":"Commands and events are processed as signals through the `Execute` module, which handles:\n\n  * Action signals (synchronous commands)\n  * Event signals (asynchronous notifications)\n  * State transitions\n  * Queue management","ref":"Jido.Agent.Server.html#module-signal-processing"},{"type":"module","title":"Fault Tolerance - Jido.Agent.Server","doc":"The server implements comprehensive error handling and cleanup:\n\n  * Graceful termination with resource cleanup\n  * Automatic unsubscription from PubSub topics\n  * Child process shutdown management\n  * Queue size monitoring and backpressure\n\nSee `cmd/4` for sending commands and `get_state/1` for retrieving current server state.","ref":"Jido.Agent.Server.html#module-fault-tolerance"},{"type":"function","title":"Jido.Agent.Server.cmd/4","doc":"Sends a command to the agent server for processing.","ref":"Jido.Agent.Server.html#cmd/4"},{"type":"function","title":"Parameters - Jido.Agent.Server.cmd/4","doc":"* `server` - The server process identifier\n  * `action` - The action module to execute\n  * `args` - Optional map of arguments for the action (default: %{})\n  * `opts` - Optional keyword list of options (default: [])","ref":"Jido.Agent.Server.html#cmd/4-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.cmd/4","doc":"* `{:ok, state}` - Command processed successfully with updated state\n  * `{:error, reason}` - Command failed with reason","ref":"Jido.Agent.Server.html#cmd/4-returns"},{"type":"macro","title":"Jido.Agent.Server.dbug/3","doc":"","ref":"Jido.Agent.Server.html#dbug/3"},{"type":"macro","title":"Jido.Agent.Server.error/3","doc":"","ref":"Jido.Agent.Server.html#error/3"},{"type":"function","title":"Jido.Agent.Server.format_status/2","doc":"Formats the server state for debugging.\n\nReturns a map containing:\n  * Current state\n  * Server status\n  * Agent ID\n  * Queue size\n  * Child process information","ref":"Jido.Agent.Server.html#format_status/2"},{"type":"function","title":"Jido.Agent.Server.get_id/1","doc":"Gets the agent ID from the server state.","ref":"Jido.Agent.Server.html#get_id/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.get_id/1","doc":"* `server` - The server process identifier","ref":"Jido.Agent.Server.html#get_id/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.get_id/1","doc":"* `{:ok, id}` - The agent ID string\n  * `{:error, reason}` - Failed to get ID","ref":"Jido.Agent.Server.html#get_id/1-returns"},{"type":"function","title":"Jido.Agent.Server.get_state/1","doc":"Gets the complete server state.","ref":"Jido.Agent.Server.html#get_state/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.get_state/1","doc":"* `server` - The server process identifier","ref":"Jido.Agent.Server.html#get_state/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.get_state/1","doc":"* `{:ok, state}` - The complete server state\n  * `{:error, reason}` - Failed to get state","ref":"Jido.Agent.Server.html#get_state/1-returns"},{"type":"function","title":"Jido.Agent.Server.get_status/1","doc":"Gets the current status from the server state.","ref":"Jido.Agent.Server.html#get_status/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.get_status/1","doc":"* `server` - The server process identifier","ref":"Jido.Agent.Server.html#get_status/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.get_status/1","doc":"* `{:ok, status}` - The current server status\n  * `{:error, reason}` - Failed to get status","ref":"Jido.Agent.Server.html#get_status/1-returns"},{"type":"function","title":"Jido.Agent.Server.get_supervisor/1","doc":"Gets the child supervisor PID from the server state.","ref":"Jido.Agent.Server.html#get_supervisor/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.get_supervisor/1","doc":"* `server` - The server process identifier","ref":"Jido.Agent.Server.html#get_supervisor/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.get_supervisor/1","doc":"* `{:ok, pid}` - The supervisor PID\n  * `{:error, reason}` - Failed to get supervisor","ref":"Jido.Agent.Server.html#get_supervisor/1-returns"},{"type":"function","title":"Jido.Agent.Server.get_topic/1","doc":"Gets the PubSub topic from the server state.","ref":"Jido.Agent.Server.html#get_topic/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.get_topic/1","doc":"* `server` - The server process identifier","ref":"Jido.Agent.Server.html#get_topic/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.get_topic/1","doc":"* `{:ok, topic}` - The PubSub topic string\n  * `{:error, reason}` - Failed to get topic","ref":"Jido.Agent.Server.html#get_topic/1-returns"},{"type":"function","title":"Jido.Agent.Server.handle_call/3","doc":"Handles synchronous calls to the server.","ref":"Jido.Agent.Server.html#handle_call/3"},{"type":"function","title":"Call Patterns - Jido.Agent.Server.handle_call/3","doc":"* `:get_state` - Returns the complete server state\n  * `%Signal{}` - Processes a signal synchronously if queue not full\n  * Other - Returns unhandled call error","ref":"Jido.Agent.Server.html#handle_call/3-call-patterns"},{"type":"function","title":"Returns - Jido.Agent.Server.handle_call/3","doc":"* `{:reply, reply, new_state}` - Response and updated state\n  * `{:reply, {:error, reason}, state}` - Error response","ref":"Jido.Agent.Server.html#handle_call/3-returns"},{"type":"function","title":"Jido.Agent.Server.handle_cast/2","doc":"Handles asynchronous casts to the server.","ref":"Jido.Agent.Server.html#handle_cast/2"},{"type":"function","title":"Cast Patterns - Jido.Agent.Server.handle_cast/2","doc":"* `%Signal{}` - Processes a signal asynchronously\n  * Other - Logs unhandled cast","ref":"Jido.Agent.Server.html#handle_cast/2-cast-patterns"},{"type":"function","title":"Returns - Jido.Agent.Server.handle_cast/2","doc":"* `{:noreply, new_state}` - Updated state after processing\n  * `{:stop, reason, state}` - Stops server on error","ref":"Jido.Agent.Server.html#handle_cast/2-returns"},{"type":"function","title":"Jido.Agent.Server.handle_info/2","doc":"Handles messages sent to the server process.","ref":"Jido.Agent.Server.html#handle_info/2"},{"type":"function","title":"Message Patterns - Jido.Agent.Server.handle_info/2","doc":"* `%Signal{}` - Processes non-event signals\n  * `:check_queue_size` - Monitors queue size and hibernates if needed\n  * `{:DOWN, ...}` - Handles child process termination\n  * `:timeout` - Handles timeout messages\n  * Other - Logs unhandled messages","ref":"Jido.Agent.Server.html#handle_info/2-message-patterns"},{"type":"function","title":"Returns - Jido.Agent.Server.handle_info/2","doc":"* `{:noreply, new_state}` - Updated state after processing\n  * `{:stop, reason, state}` - Stops server on error","ref":"Jido.Agent.Server.html#handle_info/2-returns"},{"type":"function","title":"Jido.Agent.Server.init/1","doc":"Initializes the server state with the provided configuration.","ref":"Jido.Agent.Server.html#init/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.init/1","doc":"* `%{agent:, pubsub:, topic:, max_queue_size:}` - The initialization map containing:\n    * `agent` - The agent struct to manage\n    * `pubsub` - The PubSub module for event distribution\n    * `topic` - Optional topic override (defaults to agent ID based topic)\n    * `max_queue_size` - Maximum pending signals allowed","ref":"Jido.Agent.Server.html#init/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.init/1","doc":"* `{:ok, state}` - Successfully initialized with the server state\n  * `{:stop, reason}` - Failed to initialize with error reason","ref":"Jido.Agent.Server.html#init/1-returns"},{"type":"function","title":"Jido.Agent.Server.start_link/1","doc":"Starts a new Agent Server process linked to the current process.\n\nInitializes the server with the given agent and configuration, subscribes to relevant\nPubSub topics, and starts a DynamicSupervisor for managing child processes.","ref":"Jido.Agent.Server.html#start_link/1"},{"type":"function","title":"Parameters - Jido.Agent.Server.start_link/1","doc":"* `opts` - Keyword list of server options:\n    * `:agent` - Required. Agent struct or module implementing `new/0`\n    * `:pubsub` - Required. PubSub module for event distribution\n    * `:name` - Optional. Registration name (defaults to agent.id)\n    * `:topic` - Optional. PubSub topic (defaults to generated from agent.id)\n    * `:max_queue_size` - Optional. Maximum pending signals (defaults to 10,000)\n    * `:registry` - Optional. Registry for process registration (defaults to Jido.AgentRegistry)","ref":"Jido.Agent.Server.html#start_link/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.start_link/1","doc":"* `{:ok, pid}` - Successfully started server process\n  * `{:error, reason}` - Failed to start server","ref":"Jido.Agent.Server.html#start_link/1-returns"},{"type":"function","title":"Error Reasons - Jido.Agent.Server.start_link/1","doc":"* `:invalid_agent` - Agent is nil or invalid\n  * `:missing_pubsub` - PubSub module not provided\n  * `:already_started` - Server already registered with given name\n  * Any error from PubSub subscription or DynamicSupervisor start","ref":"Jido.Agent.Server.html#start_link/1-error-reasons"},{"type":"function","title":"Examples - Jido.Agent.Server.start_link/1","doc":"# Start with minimal configuration\n    {:ok, pid} = Jido.Agent.Server.start_link([\n      agent: MyAgent.new(),\n      pubsub: MyApp.PubSub\n    ])\n\n    # Start with full configuration\n    {:ok, pid} = Jido.Agent.Server.start_link([\n      agent: MyAgent.new(),\n      pubsub: MyApp.PubSub,\n      name: \"custom_agent\",\n      topic: \"agents:custom\",\n      max_queue_size: 5_000,\n      registry: MyApp.Registry\n    ])","ref":"Jido.Agent.Server.html#start_link/1-examples"},{"type":"function","title":"Runtime Behavior - Jido.Agent.Server.start_link/1","doc":"The server performs these steps during initialization:\n1. Validates agent and builds configuration\n2. Starts DynamicSupervisor for child processes\n3. Subscribes to configured PubSub topic\n4. Transitions to :idle state\n5. Emits 'started' event\n\nThe process is registered via the configured Registry using the :via tuple\npattern for distributed process lookup.","ref":"Jido.Agent.Server.html#start_link/1-runtime-behavior"},{"type":"type","title":"Jido.Agent.Server.start_opt/0","doc":"","ref":"Jido.Agent.Server.html#t:start_opt/0"},{"type":"function","title":"Jido.Agent.Server.terminate/2","doc":"Handles cleanup when the server is terminating.\n\nPerforms the following cleanup:\n  * Emits stopped event\n  * Stops child supervisor\n  * Unsubscribes from topics","ref":"Jido.Agent.Server.html#terminate/2"},{"type":"function","title":"Parameters - Jido.Agent.Server.terminate/2","doc":"* `reason` - The reason for termination\n  * `state` - The current server state","ref":"Jido.Agent.Server.html#terminate/2-parameters"},{"type":"function","title":"Returns - Jido.Agent.Server.terminate/2","doc":"* `:ok` - Cleanup completed (even if some steps failed)","ref":"Jido.Agent.Server.html#terminate/2-returns"},{"type":"module","title":"Jido.Discovery","doc":"Discovery is the mechanism by which agents and sensors are discovered and registered with the system.\n\nThis module caches discovered components using :persistent_term for efficient lookups.\nThe cache is initialized at application startup and can be manually refreshed if needed.","ref":"Jido.Discovery.html"},{"type":"type","title":"Jido.Discovery.cache_entry/0","doc":"","ref":"Jido.Discovery.html#t:cache_entry/0"},{"type":"type","title":"Jido.Discovery.component_metadata/0","doc":"","ref":"Jido.Discovery.html#t:component_metadata/0"},{"type":"type","title":"Jido.Discovery.component_type/0","doc":"","ref":"Jido.Discovery.html#t:component_type/0"},{"type":"macro","title":"Jido.Discovery.dbug/3","doc":"","ref":"Jido.Discovery.html#dbug/3"},{"type":"macro","title":"Jido.Discovery.error/3","doc":"","ref":"Jido.Discovery.html#error/3"},{"type":"function","title":"Jido.Discovery.get_action_by_slug/1","doc":"Retrieves an Action by its slug.","ref":"Jido.Discovery.html#get_action_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_action_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Action.","ref":"Jido.Discovery.html#get_action_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_action_by_slug/1","doc":"The Action metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_action_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_action_by_slug/1","doc":"iex> Jido.get_action_by_slug(\"abc123de\")\n    %{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\"}\n\n    iex> Jido.get_action_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_action_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_agent_by_slug/1","doc":"Retrieves an Agent by its slug.","ref":"Jido.Discovery.html#get_agent_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_agent_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Agent.","ref":"Jido.Discovery.html#get_agent_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_agent_by_slug/1","doc":"The Agent metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_agent_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_agent_by_slug/1","doc":"iex> Jido.get_agent_by_slug(\"ghi789jk\")\n    %{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\"}\n\n    iex> Jido.get_agent_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_agent_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.get_sensor_by_slug/1","doc":"Retrieves a Sensor by its slug.","ref":"Jido.Discovery.html#get_sensor_by_slug/1"},{"type":"function","title":"Parameters - Jido.Discovery.get_sensor_by_slug/1","doc":"- `slug`: A string representing the unique identifier of the Sensor.","ref":"Jido.Discovery.html#get_sensor_by_slug/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.get_sensor_by_slug/1","doc":"The Sensor metadata if found, otherwise `nil`.","ref":"Jido.Discovery.html#get_sensor_by_slug/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.get_sensor_by_slug/1","doc":"iex> Jido.get_sensor_by_slug(\"def456gh\")\n    %{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\"}\n\n    iex> Jido.get_sensor_by_slug(\"nonexistent\")\n    nil","ref":"Jido.Discovery.html#get_sensor_by_slug/1-examples"},{"type":"function","title":"Jido.Discovery.init/0","doc":"Initializes the discovery cache. Should be called during application startup.","ref":"Jido.Discovery.html#init/0"},{"type":"function","title":"Returns - Jido.Discovery.init/0","doc":"- `:ok` if cache was initialized successfully\n- `{:error, reason}` if initialization failed","ref":"Jido.Discovery.html#init/0-returns"},{"type":"function","title":"Jido.Discovery.last_updated/0","doc":"Gets the last time the cache was updated.","ref":"Jido.Discovery.html#last_updated/0"},{"type":"function","title":"Returns - Jido.Discovery.last_updated/0","doc":"- `{:ok, datetime}` with the last update time\n- `{:error, :not_initialized}` if cache hasn't been initialized","ref":"Jido.Discovery.html#last_updated/0-returns"},{"type":"function","title":"Jido.Discovery.list_actions/1","doc":"Lists all Actions with optional filtering and pagination.","ref":"Jido.Discovery.html#list_actions/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_actions/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Actions by name (partial match).\n  - `:description`: Filter Actions by description (partial match).\n  - `:category`: Filter Actions by category (exact match).\n  - `:tag`: Filter Actions by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_actions/1","doc":"A list of Action metadata.","ref":"Jido.Discovery.html#list_actions/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_actions/1","doc":"iex> Jido.list_actions(limit: 10, offset: 5, category: :utility)\n    [%{module: MyApp.SomeAction, name: \"some_action\", description: \"Does something\", slug: \"abc123de\", category: :utility}]","ref":"Jido.Discovery.html#list_actions/1-examples"},{"type":"function","title":"Jido.Discovery.list_agents/1","doc":"Lists all Agents with optional filtering and pagination.","ref":"Jido.Discovery.html#list_agents/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_agents/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Agents by name (partial match).\n  - `:description`: Filter Agents by description (partial match).\n  - `:category`: Filter Agents by category (exact match).\n  - `:tag`: Filter Agents by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_agents/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_agents/1","doc":"A list of Agent metadata.","ref":"Jido.Discovery.html#list_agents/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_agents/1","doc":"iex> Jido.list_agents(limit: 10, offset: 5, category: :business)\n    [%{module: MyApp.SomeAgent, name: \"some_agent\", description: \"Represents an agent\", slug: \"ghi789jk\", category: :business}]","ref":"Jido.Discovery.html#list_agents/1-examples"},{"type":"function","title":"Jido.Discovery.list_sensors/1","doc":"Lists all Sensors with optional filtering and pagination.","ref":"Jido.Discovery.html#list_sensors/1"},{"type":"function","title":"Parameters - Jido.Discovery.list_sensors/1","doc":"- `opts`: A keyword list of options for filtering and pagination. Available options:\n  - `:limit`: Maximum number of results to return.\n  - `:offset`: Number of results to skip before starting to return.\n  - `:name`: Filter Sensors by name (partial match).\n  - `:description`: Filter Sensors by description (partial match).\n  - `:category`: Filter Sensors by category (exact match).\n  - `:tag`: Filter Sensors by tag (must have the exact tag).","ref":"Jido.Discovery.html#list_sensors/1-parameters"},{"type":"function","title":"Returns - Jido.Discovery.list_sensors/1","doc":"A list of Sensor metadata.","ref":"Jido.Discovery.html#list_sensors/1-returns"},{"type":"function","title":"Examples - Jido.Discovery.list_sensors/1","doc":"iex> Jido.list_sensors(limit: 10, offset: 5, category: :monitoring)\n    [%{module: MyApp.SomeSensor, name: \"some_sensor\", description: \"Monitors something\", slug: \"def456gh\", category: :monitoring}]","ref":"Jido.Discovery.html#list_sensors/1-examples"},{"type":"function","title":"Jido.Discovery.refresh/0","doc":"Forces a refresh of the discovery cache.","ref":"Jido.Discovery.html#refresh/0"},{"type":"function","title":"Returns - Jido.Discovery.refresh/0","doc":"- `:ok` if cache was refreshed successfully\n- `{:error, reason}` if refresh failed","ref":"Jido.Discovery.html#refresh/0-returns"},{"type":"behaviour","title":"Jido.Sensor","doc":"Defines the behavior and implementation for Sensors in the Jido system.\n\nA Sensor is a GenServer that emits Signals on PubSub based on specific events and retains a configurable number of last values.","ref":"Jido.Sensor.html"},{"type":"behaviour","title":"Usage - Jido.Sensor","doc":"To define a new Sensor, use the `Jido.Sensor` behavior in your module:\n\n    defmodule MySensor do\n      use Jido.Sensor,\n        name: \"my_sensor\",\n        description: \"Monitors a specific metric\",\n        category: :monitoring,\n        tags: [:example, :demo],\n        vsn: \"1.0.0\",\n        schema: [\n          metric: [type: :string, required: true]\n        ]\n\n      @impl true\n      def generate_signal(state) do\n        # Your sensor logic here\n        {:ok, Jido.Signal.new(%{\n          source: \"#{state.sensor.name}:#{state.id}\",\n          topic: \"metric_update\",\n          payload: %{value: get_metric_value()},\n          timestamp: DateTime.utc_now()\n        })}\n      end\n    end","ref":"Jido.Sensor.html#module-usage"},{"type":"behaviour","title":"Callbacks - Jido.Sensor","doc":"Implementing modules can override the following callbacks:\n\n- `c:mount/1`: Called when the sensor is initialized.\n- `c:generate_signal/1`: Generates a signal based on the current state.\n- `c:before_publish/2`: Called before a signal is published.\n- `c:shutdown/1`: Called when the sensor is shutting down.","ref":"Jido.Sensor.html#module-callbacks"},{"type":"callback","title":"Jido.Sensor.before_publish/2","doc":"","ref":"Jido.Sensor.html#c:before_publish/2"},{"type":"callback","title":"Jido.Sensor.generate_signal/1","doc":"","ref":"Jido.Sensor.html#c:generate_signal/1"},{"type":"callback","title":"Jido.Sensor.mount/1","doc":"","ref":"Jido.Sensor.html#c:mount/1"},{"type":"type","title":"Jido.Sensor.options/0","doc":"","ref":"Jido.Sensor.html#t:options/0"},{"type":"callback","title":"Jido.Sensor.shutdown/1","doc":"","ref":"Jido.Sensor.html#c:shutdown/1"},{"type":"type","title":"Jido.Sensor.t/0","doc":"","ref":"Jido.Sensor.html#t:t/0"},{"type":"module","title":"Jido.Signal","doc":"Defines the structure and behavior of a Signal in the Jido system.\nImplements CloudEvents specification v1.0.2 with Jido-specific extensions.","ref":"Jido.Signal.html"},{"type":"function","title":"Jido.Signal.from_map/1","doc":"Creates a new Signal struct from a map.","ref":"Jido.Signal.html#from_map/1"},{"type":"function","title":"Parameters - Jido.Signal.from_map/1","doc":"- `map`: A map containing the Signal attributes.","ref":"Jido.Signal.html#from_map/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.from_map/1","doc":"`{:ok, Signal.t()}` if the map is valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#from_map/1-returns"},{"type":"function","title":"Examples - Jido.Signal.from_map/1","doc":"iex> Jido.Signal.from_map(%{\"type\" => \"example.event\", \"source\" => \"/example\", \"id\" => \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#from_map/1-examples"},{"type":"function","title":"Jido.Signal.new/1","doc":"Creates a new Signal struct.","ref":"Jido.Signal.html#new/1"},{"type":"function","title":"Parameters - Jido.Signal.new/1","doc":"- `attrs`: A map containing the Signal attributes.","ref":"Jido.Signal.html#new/1-parameters"},{"type":"function","title":"Returns - Jido.Signal.new/1","doc":"`{:ok, Signal.t()}` if the attributes are valid, `{:error, String.t()}` otherwise.","ref":"Jido.Signal.html#new/1-returns"},{"type":"function","title":"Examples - Jido.Signal.new/1","doc":"iex> Jido.Signal.new(%{type: \"example.event\", source: \"/example\", id: \"123\"})\n    {:ok, %Jido.Signal{type: \"example.event\", source: \"/example\", id: \"123\", ...}}","ref":"Jido.Signal.html#new/1-examples"},{"type":"type","title":"Jido.Signal.t/0","doc":"","ref":"Jido.Signal.html#t:t/0"},{"type":"module","title":"Jido.Supervisor","doc":"A helper supervisor that starts and manages the resources for a specific Jido instance.\n\nEach consumer of Jido defines their own module:\n\n    defmodule MyApp.Jido do\n      use Jido, otp_app: :my_app\n    end\n\nThen in your application’s supervision tree:\n\n    children = [\n      MyApp.Jido\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one)","ref":"Jido.Supervisor.html"},{"type":"function","title":"Jido.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jido.Supervisor.html#child_spec/1"},{"type":"function","title":"Jido.Supervisor.start_link/2","doc":"","ref":"Jido.Supervisor.html#start_link/2"},{"type":"module","title":"Jido.Workflow","doc":"Workflows provide a robust framework for executing and managing workflows (Action sequences) in a distributed system.\n\nThis module offers functionality to:\n- Run workflows synchronously or asynchronously\n- Manage timeouts and retries\n- Cancel running workflows\n- Normalize and validate input parameters and context\n- Emit telemetry events for monitoring and debugging\n\nWorkflows are defined as modules (Actions) that implement specific callbacks, allowing for\na standardized way of defining and executing complex workflows across a distributed system.","ref":"Jido.Workflow.html"},{"type":"module","title":"Features - Jido.Workflow","doc":"- Synchronous and asynchronous workflow execution\n- Automatic retries with exponential backoff\n- Timeout handling for long-running workflows\n- Parameter and context normalization\n- Comprehensive error handling and reporting\n- Telemetry integration for monitoring and tracing\n- Cancellation of running workflows","ref":"Jido.Workflow.html#module-features"},{"type":"module","title":"Usage - Jido.Workflow","doc":"Workflows are executed using the `run/4` or `run_async/4` functions:\n\n    Jido.Workflow.run(MyAction, %{param1: \"value\"}, %{context_key: \"context_value\"})\n\nSee `Jido.Action` for how to define an Action.\n\nFor asynchronous execution:\n\n    async_ref = Jido.Workflow.run_async(MyAction, params, context)\n    # ... do other work ...\n    result = Jido.Workflow.await(async_ref)","ref":"Jido.Workflow.html#module-usage"},{"type":"module","title":"Integrating with OTP - Jido.Workflow","doc":"For correct supervision of async tasks, ensure you start a `Task.Supervisor` under your\napplication's supervision tree, for example:\n\n    def start(_type, _args) do\n      children = [\n        {Task.Supervisor, name: Jido.Workflow.TaskSupervisor},\n        ...\n      ]\n      Supervisor.start_link(children, strategy: :one_for_one)\n    end\n\nThis way, any async tasks spawned by `run_async/4` will be supervised by the Task Supervisor.","ref":"Jido.Workflow.html#module-integrating-with-otp"},{"type":"type","title":"Jido.Workflow.action/0","doc":"","ref":"Jido.Workflow.html#t:action/0"},{"type":"type","title":"Jido.Workflow.async_ref/0","doc":"","ref":"Jido.Workflow.html#t:async_ref/0"},{"type":"function","title":"Jido.Workflow.await/2","doc":"Waits for the result of an asynchronous Action execution.","ref":"Jido.Workflow.html#await/2"},{"type":"function","title":"Parameters - Jido.Workflow.await/2","doc":"- `async_ref`: The reference returned by `run_async/4`.\n- `timeout`: Maximum time (in ms) to wait for the result (default: 5000).","ref":"Jido.Workflow.html#await/2-parameters"},{"type":"function","title":"Returns - Jido.Workflow.await/2","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution or if the workflow times out.","ref":"Jido.Workflow.html#await/2-returns"},{"type":"function","title":"Examples - Jido.Workflow.await/2","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 10_000)\n    {:ok, %{result: \"processed value\"}}\n\n    iex> async_ref = Jido.Workflow.run_async(SlowAction, %{input: \"value\"})\n    iex> Jido.Workflow.await(async_ref, 100)\n    {:error, %Jido.Error{type: :timeout, message: \"Async workflow timed out after 100ms\"}}","ref":"Jido.Workflow.html#await/2-examples"},{"type":"function","title":"Jido.Workflow.cancel/1","doc":"Cancels a running asynchronous Action execution.","ref":"Jido.Workflow.html#cancel/1"},{"type":"function","title":"Parameters - Jido.Workflow.cancel/1","doc":"- `async_ref`: The reference returned by `run_async/4`, or just the PID of the process to cancel.","ref":"Jido.Workflow.html#cancel/1-parameters"},{"type":"function","title":"Returns - Jido.Workflow.cancel/1","doc":"- `:ok` if the cancellation was successful.\n- `{:error, reason}` if the cancellation failed or the input was invalid.","ref":"Jido.Workflow.html#cancel/1-returns"},{"type":"function","title":"Examples - Jido.Workflow.cancel/1","doc":"iex> async_ref = Jido.Workflow.run_async(LongRunningAction, %{input: \"value\"})\n    iex> Jido.Workflow.cancel(async_ref)\n    :ok\n\n    iex> Jido.Workflow.cancel(\"invalid\")\n    {:error, %Jido.Error{type: :invalid_async_ref, message: \"Invalid async ref for cancellation\"}}","ref":"Jido.Workflow.html#cancel/1-examples"},{"type":"type","title":"Jido.Workflow.context/0","doc":"","ref":"Jido.Workflow.html#t:context/0"},{"type":"type","title":"Jido.Workflow.params/0","doc":"","ref":"Jido.Workflow.html#t:params/0"},{"type":"function","title":"Jido.Workflow.run/4","doc":"Executes a Action synchronously with the given parameters and context.","ref":"Jido.Workflow.html#run/4"},{"type":"function","title":"Parameters - Jido.Workflow.run/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution:\n  - `:timeout` - Maximum time (in ms) allowed for the Action to complete (default: 5000).\n  - `:max_retries` - Maximum number of retry attempts (default: 1).\n  - `:backoff` - Initial backoff time in milliseconds, doubles with each retry (default: 1000).","ref":"Jido.Workflow.html#run/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run/4","doc":"- `{:ok, result}` if the Action executes successfully.\n- `{:error, reason}` if an error occurs during execution.","ref":"Jido.Workflow.html#run/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run/4","doc":"iex> Jido.Workflow.run(MyAction, %{input: \"value\"}, %{user_id: 123})\n    {:ok, %{result: \"processed value\"}}\n\n    iex> Jido.Workflow.run(MyAction, %{invalid: \"input\"}, %{}, timeout: 1000)\n    {:error, %Jido.Error{type: :validation_error, message: \"Invalid input\"}}","ref":"Jido.Workflow.html#run/4-examples"},{"type":"function","title":"Jido.Workflow.run_async/4","doc":"Executes a Action asynchronously with the given parameters and context.\n\nThis function immediately returns a reference that can be used to await the result\nor cancel the workflow.\n\n**Note**: This approach integrates with OTP by spawning tasks under a `Task.Supervisor`.\nMake sure `{Task.Supervisor, name: Jido.Workflow.TaskSupervisor}` is part of your supervision tree.","ref":"Jido.Workflow.html#run_async/4"},{"type":"function","title":"Parameters - Jido.Workflow.run_async/4","doc":"- `action`: The module implementing the Action behavior.\n- `params`: A map of input parameters for the Action.\n- `context`: A map providing additional context for the Action execution.\n- `opts`: Options controlling the execution (same as `run/4`).","ref":"Jido.Workflow.html#run_async/4-parameters"},{"type":"function","title":"Returns - Jido.Workflow.run_async/4","doc":"An `async_ref` map containing:\n- `:ref` - A unique reference for this async workflow.\n- `:pid` - The PID of the process executing the Action.","ref":"Jido.Workflow.html#run_async/4-returns"},{"type":"function","title":"Examples - Jido.Workflow.run_async/4","doc":"iex> async_ref = Jido.Workflow.run_async(MyAction, %{input: \"value\"}, %{user_id: 123})\n    %{ref: #Reference<0.1234.5678>, pid: #PID<0.234.0>}\n\n    iex> result = Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed value\"}}","ref":"Jido.Workflow.html#run_async/4-examples"},{"type":"type","title":"Jido.Workflow.run_opts/0","doc":"","ref":"Jido.Workflow.html#t:run_opts/0"},{"type":"module","title":"Jido.Workflow.Chain","doc":"Provides functionality to chain multiple Jido Workflows together with interruption support.\n\nThis module allows for sequential execution of workflows, where the output\nof one workflow becomes the input for the next workflow in the chain.\nExecution can be interrupted between workflows using an interruption check function.","ref":"Jido.Workflow.Chain.html"},{"type":"module","title":"Examples - Jido.Workflow.Chain","doc":"iex> interrupt_check = fn -> System.monotonic_time(:millisecond) > @deadline end\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: interrupt_check)\n    {:ok, %{value: 12}}\n\n    # When interrupted:\n    iex> Jido.Workflow.Chain.chain([AddOne, MultiplyByTwo], %{value: 5}, interrupt_check: fn -> true end)\n    {:interrupted, %{value: 6}}","ref":"Jido.Workflow.Chain.html#module-examples"},{"type":"function","title":"Jido.Workflow.Chain.chain/3","doc":"Executes a chain of workflows sequentially with optional interruption support.","ref":"Jido.Workflow.Chain.html#chain/3"},{"type":"function","title":"Parameters - Jido.Workflow.Chain.chain/3","doc":"- `workflows`: A list of workflows to be executed in order. Each workflow\n  can be a module (the workflow module) or a tuple of `{workflow_module, options}`.\n- `initial_params`: A map of initial parameters to be passed to the first workflow.\n- `opts`: Additional options for the chain execution.","ref":"Jido.Workflow.Chain.html#chain/3-parameters"},{"type":"function","title":"Options - Jido.Workflow.Chain.chain/3","doc":"- `:async` - When set to `true`, the chain will be executed asynchronously (default: `false`).\n- `:context` - A map of context data to be passed to each workflow.\n- `:interrupt_check` - A function that returns boolean, called between workflows to check if chain should be interrupted.","ref":"Jido.Workflow.Chain.html#chain/3-options"},{"type":"function","title":"Returns - Jido.Workflow.Chain.chain/3","doc":"- `{:ok, result}` where `result` is the final output of the chain.\n- `{:error, error}` if any workflow in the chain fails.\n- `{:interrupted, result}` if the chain was interrupted, containing the last successful result.\n- `Task.t()` if the `:async` option is set to `true`.","ref":"Jido.Workflow.Chain.html#chain/3-returns"},{"type":"type","title":"Jido.Workflow.Chain.chain_result/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_result/0"},{"type":"type","title":"Jido.Workflow.Chain.chain_workflow/0","doc":"","ref":"Jido.Workflow.Chain.html#t:chain_workflow/0"},{"type":"macro","title":"Jido.Workflow.Chain.dbug/3","doc":"","ref":"Jido.Workflow.Chain.html#dbug/3"},{"type":"macro","title":"Jido.Workflow.Chain.error/3","doc":"","ref":"Jido.Workflow.Chain.html#error/3"},{"type":"type","title":"Jido.Workflow.Chain.interrupt_check/0","doc":"","ref":"Jido.Workflow.Chain.html#t:interrupt_check/0"},{"type":"module","title":"Jido.Workflow.Closure","doc":"Provides functionality to create closures around Jido Workflows (Actions).\n\nThis module allows for partial application of context and options to actions,\ncreating reusable workflow closures that can be executed with different parameters.","ref":"Jido.Workflow.Closure.html"},{"type":"type","title":"Jido.Workflow.Closure.action/0","doc":"","ref":"Jido.Workflow.Closure.html#t:action/0"},{"type":"function","title":"Jido.Workflow.Closure.async_closure/3","doc":"Creates an async closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#async_closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.async_closure/3","doc":"- `action`: The action module to create an async closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#async_closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.async_closure/3","doc":"A function that takes params and returns an async reference.","ref":"Jido.Workflow.Closure.html#async_closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.async_closure/3","doc":"iex> async_closure = Jido.Workflow.Closure.async_closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> async_ref = async_closure.(%{input: \"test\"})\n    iex> Jido.Workflow.await(async_ref)\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#async_closure/3-examples"},{"type":"type","title":"Jido.Workflow.Closure.closure/0","doc":"","ref":"Jido.Workflow.Closure.html#t:closure/0"},{"type":"function","title":"Jido.Workflow.Closure.closure/3","doc":"Creates a closure around a action with pre-applied context and options.","ref":"Jido.Workflow.Closure.html#closure/3"},{"type":"function","title":"Parameters - Jido.Workflow.Closure.closure/3","doc":"- `action`: The action module to create a closure for.\n- `context`: The context to be applied to the action (default: %{}).\n- `opts`: The options to be applied to the action execution (default: []).","ref":"Jido.Workflow.Closure.html#closure/3-parameters"},{"type":"function","title":"Returns - Jido.Workflow.Closure.closure/3","doc":"A function that takes params and returns the result of running the action.","ref":"Jido.Workflow.Closure.html#closure/3-returns"},{"type":"function","title":"Examples - Jido.Workflow.Closure.closure/3","doc":"iex> closure = Jido.Workflow.Closure.closure(MyAction, %{user_id: 123}, [timeout: 10_000])\n    iex> closure.(%{input: \"test\"})\n    {:ok, %{result: \"processed test\"}}","ref":"Jido.Workflow.Closure.html#closure/3-examples"},{"type":"type","title":"Jido.Workflow.Closure.context/0","doc":"","ref":"Jido.Workflow.Closure.html#t:context/0"},{"type":"type","title":"Jido.Workflow.Closure.params/0","doc":"","ref":"Jido.Workflow.Closure.html#t:params/0"},{"type":"type","title":"Jido.Workflow.Closure.run_opts/0","doc":"","ref":"Jido.Workflow.Closure.html#t:run_opts/0"},{"type":"module","title":"Jido.Workflow.Tool","doc":"Provides functionality to convert Jido Workflows into tool representations.\n\nThis module allows Jido Workflows to be easily integrated with AI systems\nlike LangChain or Instructor by converting them into a standardized tool format.","ref":"Jido.Workflow.Tool.html"},{"type":"function","title":"Jido.Workflow.Tool.build_parameters_schema/1","doc":"Builds a parameters schema for the tool based on the workflow's schema.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.build_parameters_schema/1","doc":"* `schema` - The NimbleOptions schema from the workflow.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.build_parameters_schema/1","doc":"A map representing the parameters schema in a format compatible with LangChain.","ref":"Jido.Workflow.Tool.html#build_parameters_schema/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.execute_workflow/3","doc":"Executes an workflow and formats the result for tool output.\n\nThis function is typically used as the function value in the tool representation.","ref":"Jido.Workflow.Tool.html#execute_workflow/3"},{"type":"function","title":"Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"Converts a NimbleOptions type to a JSON Schema type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"* `type` - The NimbleOptions type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.nimble_type_to_json_schema_type/1","doc":"A string representing the equivalent JSON Schema type.","ref":"Jido.Workflow.Tool.html#nimble_type_to_json_schema_type/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"Converts a NimbleOptions parameter definition to a JSON Schema representation.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"* `opts` - The options for a single parameter from the NimbleOptions schema.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.parameter_to_json_schema/1","doc":"A map representing the parameter in JSON Schema format.","ref":"Jido.Workflow.Tool.html#parameter_to_json_schema/1-returns"},{"type":"function","title":"Jido.Workflow.Tool.to_tool/1","doc":"Converts a Jido Workflow into a tool representation.","ref":"Jido.Workflow.Tool.html#to_tool/1"},{"type":"function","title":"Arguments - Jido.Workflow.Tool.to_tool/1","doc":"* `workflow` - The module implementing the Jido.Action behavior.","ref":"Jido.Workflow.Tool.html#to_tool/1-arguments"},{"type":"function","title":"Returns - Jido.Workflow.Tool.to_tool/1","doc":"A map representing the workflow as a tool, compatible with systems like LangChain.","ref":"Jido.Workflow.Tool.html#to_tool/1-returns"},{"type":"function","title":"Examples - Jido.Workflow.Tool.to_tool/1","doc":"iex> tool = Jido.Workflow.Tool.to_tool(MyWorkflow)\n    %{\n      name: \"my_workflow\",\n      description: \"Performs a specific task\",\n      function: #Function<...>,\n      parameters_schema: %{...}\n    }","ref":"Jido.Workflow.Tool.html#to_tool/1-examples"},{"type":"type","title":"Jido.Workflow.Tool.tool/0","doc":"","ref":"Jido.Workflow.Tool.html#t:tool/0"},{"type":"module","title":"Jido.Action.Directives","doc":"A collection of actions that are used to control the behavior of the agent.\n\nThis module provides a set of simple, reusable actions:\n- EnqueueAction: Enqueues another action based on params\n- RegisterAction: Registers a new action module\n- DeregisterAction: Deregisters an existing action module\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Action.Directives.html"},{"type":"module","title":"Jido.Actions.Basic","doc":"A collection of basic actions for common workflows.\n\nThis module provides a set of simple, reusable actions:\n- Sleep: Pauses execution for a specified duration\n- Log: Logs a message with a specified level\n- Todo: Logs a todo item as a placeholder or reminder\n- RandomSleep: Introduces a random delay within a specified range\n- Increment: Increments a value by 1\n- Decrement: Decrements a value by 1\n- Noop: No operation, returns input unchanged\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Basic.html"},{"type":"module","title":"Jido.Actions.Calculator","doc":"Provides basic calculator operations as actions. The operations defined here are obviously trivial.\nThis module is intended to be used as a reference for how to write your own actions.\n\nThis module defines a set of calculator actions that can be used in workflows:\n- Add: Adds two numbers together\n- Subtract: Subtracts one number from another\n- Multiply: Multiplies two numbers\n- Divide: Divides one number by another, handling division by zero\n- Square: Squares a number\n\nEach action is implemented as a separate submodule and follows the Jido.Action behavior.","ref":"Jido.Actions.Calculator.html"},{"type":"module","title":"Jido.Actions.Files","doc":"Comprehensive file manipulation actions for the Jido SDK.\n\nThis module provides actions for:\n- File operations (read, write, copy, move, delete)\n- Directory operations (create, remove, list)\n- Path manipulation and verification\n- File metadata operations\n- Content manipulation","ref":"Jido.Actions.Files.html"},{"type":"module","title":"Jido.Actions.Files.DeleteFile","doc":"Deletes a file or directory","ref":"Jido.Actions.Files.DeleteFile.html"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.category/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#category/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.description/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#description/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.name/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#name/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.on_after_run/1","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#on_after_run/1"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.on_after_validate_params/1","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#on_after_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.on_before_validate_params/1","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#on_before_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.on_error/4","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#on_error/4"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.run/2","doc":"Executes the Action with the given parameters and context.\n\nThe `run/2` function must be implemented in the module using Jido.Action.","ref":"Jido.Actions.Files.DeleteFile.html#run/2"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.schema/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#schema/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.tags/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#tags/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.to_json/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#to_json/0"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.validate_params/1","doc":"Validates the input parameters for the Action.","ref":"Jido.Actions.Files.DeleteFile.html#validate_params/1"},{"type":"function","title":"Examples - Jido.Actions.Files.DeleteFile.validate_params/1","doc":"iex> defmodule ExampleAction do\n    ...>   use Jido.Action,\n    ...>     name: \"example_action\",\n    ...>     schema: [\n    ...>       input: [type: :string, required: true]\n    ...>     ]\n    ...> end\n    ...> ExampleAction.validate_params(%{input: \"test\"})\n    {:ok, %{input: \"test\"}}\n\n    iex> ExampleAction.validate_params(%{})\n    {:error, \"Invalid parameters for Action: Required key :input not found\"}","ref":"Jido.Actions.Files.DeleteFile.html#validate_params/1-examples"},{"type":"function","title":"Jido.Actions.Files.DeleteFile.vsn/0","doc":"","ref":"Jido.Actions.Files.DeleteFile.html#vsn/0"},{"type":"module","title":"Jido.Actions.Files.ListDirectory","doc":"Lists contents of a directory with optional filtering","ref":"Jido.Actions.Files.ListDirectory.html"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.category/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#category/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.description/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#description/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.name/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#name/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.on_after_run/1","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#on_after_run/1"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.on_after_validate_params/1","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#on_after_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.on_before_validate_params/1","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#on_before_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.on_error/4","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#on_error/4"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.run/2","doc":"Executes the Action with the given parameters and context.\n\nThe `run/2` function must be implemented in the module using Jido.Action.","ref":"Jido.Actions.Files.ListDirectory.html#run/2"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.schema/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#schema/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.tags/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#tags/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.to_json/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#to_json/0"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.validate_params/1","doc":"Validates the input parameters for the Action.","ref":"Jido.Actions.Files.ListDirectory.html#validate_params/1"},{"type":"function","title":"Examples - Jido.Actions.Files.ListDirectory.validate_params/1","doc":"iex> defmodule ExampleAction do\n    ...>   use Jido.Action,\n    ...>     name: \"example_action\",\n    ...>     schema: [\n    ...>       input: [type: :string, required: true]\n    ...>     ]\n    ...> end\n    ...> ExampleAction.validate_params(%{input: \"test\"})\n    {:ok, %{input: \"test\"}}\n\n    iex> ExampleAction.validate_params(%{})\n    {:error, \"Invalid parameters for Action: Required key :input not found\"}","ref":"Jido.Actions.Files.ListDirectory.html#validate_params/1-examples"},{"type":"function","title":"Jido.Actions.Files.ListDirectory.vsn/0","doc":"","ref":"Jido.Actions.Files.ListDirectory.html#vsn/0"},{"type":"module","title":"Jido.Actions.Files.MakeDirectory","doc":"Creates a directory and optionally its parent directories","ref":"Jido.Actions.Files.MakeDirectory.html"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.category/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#category/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.description/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#description/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.name/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#name/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.on_after_run/1","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#on_after_run/1"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.on_after_validate_params/1","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#on_after_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.on_before_validate_params/1","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#on_before_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.on_error/4","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#on_error/4"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.run/2","doc":"Executes the Action with the given parameters and context.\n\nThe `run/2` function must be implemented in the module using Jido.Action.","ref":"Jido.Actions.Files.MakeDirectory.html#run/2"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.schema/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#schema/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.tags/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#tags/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.to_json/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#to_json/0"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.validate_params/1","doc":"Validates the input parameters for the Action.","ref":"Jido.Actions.Files.MakeDirectory.html#validate_params/1"},{"type":"function","title":"Examples - Jido.Actions.Files.MakeDirectory.validate_params/1","doc":"iex> defmodule ExampleAction do\n    ...>   use Jido.Action,\n    ...>     name: \"example_action\",\n    ...>     schema: [\n    ...>       input: [type: :string, required: true]\n    ...>     ]\n    ...> end\n    ...> ExampleAction.validate_params(%{input: \"test\"})\n    {:ok, %{input: \"test\"}}\n\n    iex> ExampleAction.validate_params(%{})\n    {:error, \"Invalid parameters for Action: Required key :input not found\"}","ref":"Jido.Actions.Files.MakeDirectory.html#validate_params/1-examples"},{"type":"function","title":"Jido.Actions.Files.MakeDirectory.vsn/0","doc":"","ref":"Jido.Actions.Files.MakeDirectory.html#vsn/0"},{"type":"module","title":"Jido.Actions.Files.WriteFile","doc":"Writes content to a file, creating parent directories if needed","ref":"Jido.Actions.Files.WriteFile.html"},{"type":"function","title":"Jido.Actions.Files.WriteFile.category/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#category/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.description/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#description/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.name/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#name/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.on_after_run/1","doc":"","ref":"Jido.Actions.Files.WriteFile.html#on_after_run/1"},{"type":"function","title":"Jido.Actions.Files.WriteFile.on_after_validate_params/1","doc":"","ref":"Jido.Actions.Files.WriteFile.html#on_after_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.WriteFile.on_before_validate_params/1","doc":"","ref":"Jido.Actions.Files.WriteFile.html#on_before_validate_params/1"},{"type":"function","title":"Jido.Actions.Files.WriteFile.on_error/4","doc":"","ref":"Jido.Actions.Files.WriteFile.html#on_error/4"},{"type":"function","title":"Jido.Actions.Files.WriteFile.run/2","doc":"Executes the Action with the given parameters and context.\n\nThe `run/2` function must be implemented in the module using Jido.Action.","ref":"Jido.Actions.Files.WriteFile.html#run/2"},{"type":"function","title":"Jido.Actions.Files.WriteFile.schema/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#schema/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.tags/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#tags/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.to_json/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#to_json/0"},{"type":"function","title":"Jido.Actions.Files.WriteFile.validate_params/1","doc":"Validates the input parameters for the Action.","ref":"Jido.Actions.Files.WriteFile.html#validate_params/1"},{"type":"function","title":"Examples - Jido.Actions.Files.WriteFile.validate_params/1","doc":"iex> defmodule ExampleAction do\n    ...>   use Jido.Action,\n    ...>     name: \"example_action\",\n    ...>     schema: [\n    ...>       input: [type: :string, required: true]\n    ...>     ]\n    ...> end\n    ...> ExampleAction.validate_params(%{input: \"test\"})\n    {:ok, %{input: \"test\"}}\n\n    iex> ExampleAction.validate_params(%{})\n    {:error, \"Invalid parameters for Action: Required key :input not found\"}","ref":"Jido.Actions.Files.WriteFile.html#validate_params/1-examples"},{"type":"function","title":"Jido.Actions.Files.WriteFile.vsn/0","doc":"","ref":"Jido.Actions.Files.WriteFile.html#vsn/0"},{"type":"module","title":"Jido.Actions.Simplebot","doc":"A collection of actions for a simple robot simulation. These are intended to be used as an example and reference for how to write your own bot logic.\n\nThis module provides actions for:\n- Move: Simulates moving the robot to a specified location\n- Idle: Simulates the robot idling\n- DoWork: Simulates the robot performing work tasks\n- Report: Simulates the robot reporting its status\n- Recharge: Simulates recharging the robot's battery","ref":"Jido.Actions.Simplebot.html"},{"type":"module","title":"Jido.Actions.Syscall","doc":"Actions for executing system calls from within agent workflows.\nThese actions provide controlled access to server operations.","ref":"Jido.Actions.Syscall.html"},{"type":"module","title":"Jido.Agent.Directive","doc":"Provides a type-safe way to modify agent state through discrete, validated directives.","ref":"Jido.Agent.Directive.html"},{"type":"module","title":"Overview - Jido.Agent.Directive","doc":"Directives are immutable instructions that can be applied to an agent to modify its state\n  in predefined ways. Each directive type is implemented as a separate struct with its own\n  validation rules, helping ensure type safety and consistent state transitions.","ref":"Jido.Agent.Directive.html#module-overview"},{"type":"module","title":"Available Directives - Jido.Agent.Directive","doc":"* `EnqueueDirective` - Adds a new instruction to the agent's pending queue\n      - Requires an action atom\n      - Supports optional params and context maps\n      - Example: `%EnqueueDirective{action: :move, params: %{location: :kitchen}}`\n\n  * `RegisterActionDirective` - Registers a new action module with the agent\n      - Requires a valid module atom\n      - Example: `%RegisterActionDirective{action_module: MyApp.Actions.Move}`\n\n  * `DeregisterActionDirective` - Removes an action module from the agent\n      - Requires a valid module atom\n      - Example: `%DeregisterActionDirective{action_module: MyApp.Actions.Move}`","ref":"Jido.Agent.Directive.html#module-available-directives"},{"type":"module","title":"Usage - Jido.Agent.Directive","doc":"Directives are typically created by action handlers and applied through the `apply_directives/3`\n  function. The function processes directives in order and ensures atomicity - if any directive\n  fails, the entire operation is rolled back.\n\n  ```elixir\n  # Single directive\n  directive = %EnqueueDirective{action: :move, params: %{location: :kitchen}}\n  result = %Result{directives: [directive]}\n  {:ok, updated_agent} = Directive.apply_directives(agent, result)\n\n  # Multiple directives\n  directives = [\n    %RegisterActionDirective{action_module: MyApp.Actions.Move},\n    %EnqueueDirective{action: :move, params: %{location: :kitchen}}\n  ]\n\n  result = %Result{directives: directives}\n  {:ok, updated_agent} = Directive.apply_directives(agent, result)\n  ```","ref":"Jido.Agent.Directive.html#module-usage"},{"type":"module","title":"Validation - Jido.Agent.Directive","doc":"Each directive type has its own validation rules:\n\n  * `EnqueueDirective` requires a non-nil atom for the action\n  * `RegisterActionDirective` requires a valid module atom\n  * `DeregisterActionDirective` requires a valid module atom\n\n  Failed validation results in an error tuple being returned and processing being halted.","ref":"Jido.Agent.Directive.html#module-validation"},{"type":"module","title":"Error Handling - Jido.Agent.Directive","doc":"The module uses tagged tuples for error handling:\n\n  * `{:ok, updated_agent}` - Successful application of directives\n  * `{:error, reason}` - Failed validation or application\n\n  Common error reasons include:\n\n  * `:invalid_action` - The action specified in an `EnqueueDirective` is invalid\n  * `:invalid_action_module` - The module specified in a `Register/DeregisterDirective` is invalid","ref":"Jido.Agent.Directive.html#module-error-handling"},{"type":"function","title":"Jido.Agent.Directive.apply_directive/3","doc":"Applies a single directive to an agent. Pattern matches on directive type\nto execute the appropriate transformation.","ref":"Jido.Agent.Directive.html#apply_directive/3"},{"type":"function","title":"Parameters - Jido.Agent.Directive.apply_directive/3","doc":"- agent: The agent struct to apply the directive to\n  - directive: The directive struct to apply\n  - opts: Optional keyword list of options (default: [])","ref":"Jido.Agent.Directive.html#apply_directive/3-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.apply_directive/3","doc":"- `{:ok, updated_agent}` - Directive was successfully applied\n  - `{:error, reason}` - Failed to apply directive with reason","ref":"Jido.Agent.Directive.html#apply_directive/3-returns"},{"type":"function","title":"Directive Types - Jido.Agent.Directive.apply_directive/3","doc":"","ref":"Jido.Agent.Directive.html#apply_directive/3-directive-types"},{"type":"function","title":"EnqueueDirective - Jido.Agent.Directive.apply_directive/3","doc":"Adds a new instruction to the agent's pending queue.","ref":"Jido.Agent.Directive.html#apply_directive/3-enqueuedirective"},{"type":"function","title":"RegisterActionDirective - Jido.Agent.Directive.apply_directive/3","doc":"Registers a new action module with the agent.","ref":"Jido.Agent.Directive.html#apply_directive/3-registeractiondirective"},{"type":"function","title":"DeregisterActionDirective - Jido.Agent.Directive.apply_directive/3","doc":"Removes an action module from the agent.","ref":"Jido.Agent.Directive.html#apply_directive/3-deregisteractiondirective"},{"type":"function","title":"Jido.Agent.Directive.apply_directives/3","doc":"Applies a list of directives to an agent, maintaining ordering and atomicity.\nReturns either the updated agent or an error if any directive application fails.","ref":"Jido.Agent.Directive.html#apply_directives/3"},{"type":"function","title":"Parameters - Jido.Agent.Directive.apply_directives/3","doc":"- agent: The agent struct to apply directives to\n  - result: A Result struct containing the list of directives to apply\n  - opts: Optional keyword list of options (default: [])","ref":"Jido.Agent.Directive.html#apply_directives/3-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.apply_directives/3","doc":"- `{:ok, updated_agent}` - All directives were successfully applied\n  - `{:error, reason}` - A directive failed to apply, with reason for failure","ref":"Jido.Agent.Directive.html#apply_directives/3-returns"},{"type":"function","title":"Examples - Jido.Agent.Directive.apply_directives/3","doc":"result = %Result{directives: [\n      %EnqueueDirective{action: :my_action, params: %{key: \"value\"}},\n      %RegisterActionDirective{action_module: MyAction}\n    ]}\n\n    {:ok, updated_agent} = Directive.apply_directives(agent, result)","ref":"Jido.Agent.Directive.html#apply_directives/3-examples"},{"type":"function","title":"Behavior - Jido.Agent.Directive.apply_directives/3","doc":"- Applies directives in order, stopping on first error\n- Maintains atomicity - all directives succeed or none are applied\n- Logs debug info about directive application","ref":"Jido.Agent.Directive.html#apply_directives/3-behavior"},{"type":"macro","title":"Jido.Agent.Directive.dbug/3","doc":"","ref":"Jido.Agent.Directive.html#dbug/3"},{"type":"type","title":"Jido.Agent.Directive.directive_result/0","doc":"","ref":"Jido.Agent.Directive.html#t:directive_result/0"},{"type":"macro","title":"Jido.Agent.Directive.error/3","doc":"","ref":"Jido.Agent.Directive.html#error/3"},{"type":"function","title":"Jido.Agent.Directive.is_directive?/1","doc":"Checks if a value is a valid directive struct or ok-tupled directive.\n\nA valid directive is either:\n- A struct of type EnqueueDirective, RegisterActionDirective, or DeregisterActionDirective\n- An ok-tuple containing one of the above directive structs","ref":"Jido.Agent.Directive.html#is_directive?/1"},{"type":"function","title":"Parameters - Jido.Agent.Directive.is_directive?/1","doc":"- value: Any value to check","ref":"Jido.Agent.Directive.html#is_directive?/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Directive.is_directive?/1","doc":"- `true` if the value is a valid directive\n  - `false` otherwise","ref":"Jido.Agent.Directive.html#is_directive?/1-returns"},{"type":"function","title":"Examples - Jido.Agent.Directive.is_directive?/1","doc":"iex> is_directive?(%EnqueueDirective{action: :test})\n    true\n\n    iex> is_directive?({:ok, %RegisterActionDirective{action_module: MyModule}})\n    true\n\n    iex> is_directive?(:not_a_directive)\n    false","ref":"Jido.Agent.Directive.html#is_directive?/1-examples"},{"type":"type","title":"Jido.Agent.Directive.t/0","doc":"","ref":"Jido.Agent.Directive.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.DeregisterActionDirective","doc":"","ref":"Jido.Agent.Directive.DeregisterActionDirective.html"},{"type":"type","title":"Jido.Agent.Directive.DeregisterActionDirective.t/0","doc":"Directive to deregister an existing action module","ref":"Jido.Agent.Directive.DeregisterActionDirective.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.EnqueueDirective","doc":"","ref":"Jido.Agent.Directive.EnqueueDirective.html"},{"type":"type","title":"Jido.Agent.Directive.EnqueueDirective.t/0","doc":"Directive to enqueue a new instruction","ref":"Jido.Agent.Directive.EnqueueDirective.html#t:t/0"},{"type":"module","title":"Jido.Agent.Directive.RegisterActionDirective","doc":"","ref":"Jido.Agent.Directive.RegisterActionDirective.html"},{"type":"type","title":"Jido.Agent.Directive.RegisterActionDirective.t/0","doc":"Directive to register a new action module","ref":"Jido.Agent.Directive.RegisterActionDirective.html#t:t/0"},{"type":"module","title":"Jido.Agent.Syscall","doc":"Provides a type-safe way to execute system commands that affect agent server behavior.","ref":"Jido.Agent.Syscall.html"},{"type":"module","title":"Overview - Jido.Agent.Syscall","doc":"Syscalls are immutable instructions that can be executed by the agent server to perform\nsystem-level operations like spawning processes, broadcasting messages, etc. Each syscall\ntype is implemented as a separate struct with its own validation rules.","ref":"Jido.Agent.Syscall.html#module-overview"},{"type":"module","title":"Available Syscalls - Jido.Agent.Syscall","doc":"* `SpawnSyscall` - Spawns a child process under the agent's supervisor\n    - Requires a module atom and arguments\n    - Example: `%SpawnSyscall{module: MyWorker, args: [id: 1]}`\n\n* `KillSyscall` - Terminates a child process\n    - Requires a valid PID\n    - Example: `%KillSyscall{pid: #PID<0.123.0>}`\n\n* `BroadcastSyscall` - Broadcasts a message on a PubSub topic\n    - Requires a topic string and message\n    - Example: `%BroadcastSyscall{topic: \"events\", message: %{type: :update}}`\n\n* `SubscribeSyscall` - Subscribes to a PubSub topic\n    - Requires a topic string\n    - Example: `%SubscribeSyscall{topic: \"events\"}`\n\n* `UnsubscribeSyscall` - Unsubscribes from a PubSub topic\n    - Requires a topic string\n    - Example: `%UnsubscribeSyscall{topic: \"events\"}`","ref":"Jido.Agent.Syscall.html#module-available-syscalls"},{"type":"module","title":"Validation - Jido.Agent.Syscall","doc":"Each syscall type has its own validation rules to ensure type safety and valid parameters.\nFailed validation results in an error tuple being returned.","ref":"Jido.Agent.Syscall.html#module-validation"},{"type":"function","title":"Jido.Agent.Syscall.is_syscall?/1","doc":"Checks if a value is a valid syscall struct.","ref":"Jido.Agent.Syscall.html#is_syscall?/1"},{"type":"function","title":"Parameters - Jido.Agent.Syscall.is_syscall?/1","doc":"- value: Any value to check","ref":"Jido.Agent.Syscall.html#is_syscall?/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Syscall.is_syscall?/1","doc":"- `true` if the value is a valid syscall struct\n  - `false` otherwise","ref":"Jido.Agent.Syscall.html#is_syscall?/1-returns"},{"type":"function","title":"Examples - Jido.Agent.Syscall.is_syscall?/1","doc":"iex> is_syscall?(%SpawnSyscall{module: MyWorker, args: []})\n    true\n\n    iex> is_syscall?(:not_a_syscall)\n    false","ref":"Jido.Agent.Syscall.html#is_syscall?/1-examples"},{"type":"type","title":"Jido.Agent.Syscall.t/0","doc":"","ref":"Jido.Agent.Syscall.html#t:t/0"},{"type":"function","title":"Jido.Agent.Syscall.validate_syscall/1","doc":"Validates a syscall struct based on its type.","ref":"Jido.Agent.Syscall.html#validate_syscall/1"},{"type":"function","title":"Parameters - Jido.Agent.Syscall.validate_syscall/1","doc":"- syscall: The syscall struct to validate","ref":"Jido.Agent.Syscall.html#validate_syscall/1-parameters"},{"type":"function","title":"Returns - Jido.Agent.Syscall.validate_syscall/1","doc":"- `:ok` if validation passes\n  - `{:error, reason}` if validation fails","ref":"Jido.Agent.Syscall.html#validate_syscall/1-returns"},{"type":"function","title":"Examples - Jido.Agent.Syscall.validate_syscall/1","doc":"iex> validate_syscall(%SpawnSyscall{module: MyWorker, args: []})\n    :ok\n\n    iex> validate_syscall(%SpawnSyscall{module: nil, args: []})\n    {:error, :invalid_module}","ref":"Jido.Agent.Syscall.html#validate_syscall/1-examples"},{"type":"module","title":"Jido.Agent.Syscall.BroadcastSyscall","doc":"","ref":"Jido.Agent.Syscall.BroadcastSyscall.html"},{"type":"type","title":"Jido.Agent.Syscall.BroadcastSyscall.t/0","doc":"Syscall to broadcast a message","ref":"Jido.Agent.Syscall.BroadcastSyscall.html#t:t/0"},{"type":"module","title":"Jido.Agent.Syscall.KillSyscall","doc":"","ref":"Jido.Agent.Syscall.KillSyscall.html"},{"type":"type","title":"Jido.Agent.Syscall.KillSyscall.t/0","doc":"Syscall to terminate a child process","ref":"Jido.Agent.Syscall.KillSyscall.html#t:t/0"},{"type":"module","title":"Jido.Agent.Syscall.SpawnSyscall","doc":"","ref":"Jido.Agent.Syscall.SpawnSyscall.html"},{"type":"type","title":"Jido.Agent.Syscall.SpawnSyscall.t/0","doc":"Syscall to spawn a child process","ref":"Jido.Agent.Syscall.SpawnSyscall.html#t:t/0"},{"type":"module","title":"Jido.Agent.Syscall.SubscribeSyscall","doc":"","ref":"Jido.Agent.Syscall.SubscribeSyscall.html"},{"type":"type","title":"Jido.Agent.Syscall.SubscribeSyscall.t/0","doc":"Syscall to subscribe to a topic","ref":"Jido.Agent.Syscall.SubscribeSyscall.html#t:t/0"},{"type":"module","title":"Jido.Agent.Syscall.UnsubscribeSyscall","doc":"","ref":"Jido.Agent.Syscall.UnsubscribeSyscall.html"},{"type":"type","title":"Jido.Agent.Syscall.UnsubscribeSyscall.t/0","doc":"Syscall to unsubscribe from a topic","ref":"Jido.Agent.Syscall.UnsubscribeSyscall.html#t:t/0"},{"type":"behaviour","title":"Jido.Runner","doc":"Behavior for executing planned actions on an Agent.","ref":"Jido.Runner.html"},{"type":"type","title":"Jido.Runner.action/0","doc":"","ref":"Jido.Runner.html#t:action/0"},{"type":"function","title":"Jido.Runner.normalize_instructions/1","doc":"Normalizes actions into a list of instruction tuples {module, map()}.\n\nAccepts:\n• A single action module\n• A single action tuple {module, map()}\n• A list of modules\n• A list of action tuples {module, map()}","ref":"Jido.Runner.html#normalize_instructions/1"},{"type":"function","title":"Examples - Jido.Runner.normalize_instructions/1","doc":"iex> normalize_instructions(MyAction)\n    [{MyAction, %{}}]\n\n    iex> normalize_instructions({MyAction, %{foo: :bar}})\n    [{MyAction, %{foo: :bar}}]\n\n    iex> normalize_instructions([MyAction1, MyAction2])\n    [{MyAction1, %{}}, {MyAction2, %{}}]\n\n    iex> normalize_instructions([{MyAction1, %{a: 1}}, {MyAction2, %{b: 2}}])\n    [{MyAction1, %{a: 1}}, {MyAction2, %{b: 2}}]","ref":"Jido.Runner.html#normalize_instructions/1-examples"},{"type":"callback","title":"Jido.Runner.run/2","doc":"","ref":"Jido.Runner.html#c:run/2"},{"type":"module","title":"Jido.Runner.Chain","doc":"A runner that executes instructions sequentially with support for result chaining,\ndirective-based interruption, and syscall handling.","ref":"Jido.Runner.Chain.html"},{"type":"module","title":"Chain Execution - Jido.Runner.Chain","doc":"Instructions are executed in sequence with the output of each instruction\nbecoming the input for the next instruction in the chain. This enables\ndata flow between instructions while maintaining state consistency.","ref":"Jido.Runner.Chain.html#module-chain-execution"},{"type":"module","title":"Directive Handling - Jido.Runner.Chain","doc":"The chain supports directive-based flow control:\n* Directives can interrupt chain execution (default)\n* Directives can be collected while continuing execution\n* State changes and directives can be mixed in the chain","ref":"Jido.Runner.Chain.html#module-directive-handling"},{"type":"module","title":"Syscall Handling - Jido.Runner.Chain","doc":"* Syscalls are special instructions that affect server behavior\n* Syscalls are validated before execution\n* Syscalls can be mixed with regular instructions\n* Chain execution continues after syscall handling","ref":"Jido.Runner.Chain.html#module-syscall-handling"},{"type":"module","title":"State Management - Jido.Runner.Chain","doc":"* Initial state flows through the chain\n* Each instruction can modify or extend the state\n* Final state reflects accumulated changes\n* Directive interrupts preserve current state","ref":"Jido.Runner.Chain.html#module-state-management"},{"type":"type","title":"Jido.Runner.Chain.chain_opts/0","doc":"","ref":"Jido.Runner.Chain.html#t:chain_opts/0"},{"type":"type","title":"Jido.Runner.Chain.chain_result/0","doc":"","ref":"Jido.Runner.Chain.html#t:chain_result/0"},{"type":"macro","title":"Jido.Runner.Chain.dbug/3","doc":"","ref":"Jido.Runner.Chain.html#dbug/3"},{"type":"macro","title":"Jido.Runner.Chain.error/3","doc":"","ref":"Jido.Runner.Chain.html#error/3"},{"type":"function","title":"Jido.Runner.Chain.run/2","doc":"Executes a chain of instructions, handling directives, syscalls and state transitions.","ref":"Jido.Runner.Chain.html#run/2"},{"type":"function","title":"Execution Flow - Jido.Runner.Chain.run/2","doc":"1. Instructions are executed in sequence\n2. Results from each instruction feed into the next\n3. Directives can interrupt or continue the chain\n4. Syscalls are validated and handled\n5. State changes are accumulated through the chain","ref":"Jido.Runner.Chain.html#run/2-execution-flow"},{"type":"function","title":"Parameters - Jido.Runner.Chain.run/2","doc":"- agent: The agent struct containing pending instructions\n  - opts: Optional keyword list of execution options:\n    - :continue_on_directive - boolean, continues chain execution after directive (default: false)\n    - :merge_results - boolean, merges map results into subsequent instruction params (default: true)","ref":"Jido.Runner.Chain.html#run/2-parameters"},{"type":"function","title":"Returns - Jido.Runner.Chain.run/2","doc":"- `{:ok, %Result{}}` - Chain completed successfully\n    - result_state: Final accumulated state\n    - directives: List of encountered directives\n    - syscalls: List of executed syscalls\n    - status: Final execution status\n  - `{:error, term()}` - Chain execution failed\n    - Contains error details and partial results","ref":"Jido.Runner.Chain.html#run/2-returns"},{"type":"function","title":"Examples - Jido.Runner.Chain.run/2","doc":"# Basic chain execution\n    {:ok, result} = Chain.run(agent)\n\n    # Continue after directives\n    {:ok, result} = Chain.run(agent, continue_on_directive: true)\n\n    # Handle execution error\n    {:error, error} = Chain.run(agent_with_failing_instruction)","ref":"Jido.Runner.Chain.html#run/2-examples"},{"type":"module","title":"Jido.Runner.Instruction","doc":"Represents a single instruction to be executed by a Runner.\nAn instruction consists of an action module and its parameters.\n\nInstructions are the basic unit of execution in the Jido system. They wrap an action module\nwith its parameters and execution context, allowing the Runner to execute them in a\nstandardized way.","ref":"Jido.Runner.Instruction.html"},{"type":"module","title":"Fields - Jido.Runner.Instruction","doc":"- `:action` - The action module to execute (required)\n- `:params` - Map of parameters to pass to the action (default: %{})\n- `:context` - Map of execution context data (default: %{})\n- `:result` - Result of executing the instruction (default: nil)","ref":"Jido.Runner.Instruction.html#module-fields"},{"type":"module","title":"Examples - Jido.Runner.Instruction","doc":"# Create a basic instruction\n    %Instruction{\n      action: MyApp.Actions.DoSomething,\n      params: %{value: 42}\n    }\n\n    # With context\n    %Instruction{\n      action: MyApp.Actions.ProcessData,\n      params: %{data: \"input\"},\n      context: %{user_id: 123}\n    }\n\nInstructions are typically created by the Agent when processing commands, and then\nexecuted by a Runner module like `Jido.Runner.Simple` or `Jido.Runner.Chain`.","ref":"Jido.Runner.Instruction.html#module-examples"},{"type":"type","title":"Jido.Runner.Instruction.t/0","doc":"","ref":"Jido.Runner.Instruction.html#t:t/0"},{"type":"module","title":"Jido.Runner.Result","doc":"Represents the result of executing one or more instructions.\nContains the final state and any directives for the agent.\n\nThe Result struct tracks:\n- Initial and final state of the execution\n- Status and any errors that occurred\n- Instructions that were executed\n- Any directives generated during execution\n- Remaining pending instructions\n\nThis is used by both the Simple and Chain runners to maintain execution state\nand return results in a consistent format.","ref":"Jido.Runner.Result.html"},{"type":"type","title":"Jido.Runner.Result.t/0","doc":"","ref":"Jido.Runner.Result.html#t:t/0"},{"type":"module","title":"Jido.Runner.Simple","doc":"A simple runner that executes a single instruction from an Agent's instruction queue.","ref":"Jido.Runner.Simple.html"},{"type":"module","title":"Overview - Jido.Runner.Simple","doc":"The Simple Runner follows a sequential execution model:\n1. Dequeues a single instruction from the agent's pending queue\n2. Executes the instruction via its action module\n3. Processes the result (either a directive, syscall or state update)\n4. Returns a Result struct containing the execution outcome","ref":"Jido.Runner.Simple.html#module-overview"},{"type":"module","title":"Features - Jido.Runner.Simple","doc":"* Single instruction execution\n* Support for directives, syscalls and state results\n* Atomic execution guarantees\n* Comprehensive error handling\n* Debug logging at key execution points","ref":"Jido.Runner.Simple.html#module-features"},{"type":"module","title":"Error Handling - Jido.Runner.Simple","doc":"* Invalid instructions are rejected\n* Action execution failures return error results\n* Queue empty condition handled gracefully\n* All errors preserve the original agent state","ref":"Jido.Runner.Simple.html#module-error-handling"},{"type":"macro","title":"Jido.Runner.Simple.dbug/3","doc":"","ref":"Jido.Runner.Simple.html#dbug/3"},{"type":"macro","title":"Jido.Runner.Simple.error/3","doc":"","ref":"Jido.Runner.Simple.html#error/3"},{"type":"function","title":"Jido.Runner.Simple.run/2","doc":"Executes a single instruction from the Agent's pending instructions queue.","ref":"Jido.Runner.Simple.html#run/2"},{"type":"function","title":"Execution Process - Jido.Runner.Simple.run/2","doc":"1. Dequeues the oldest instruction from the agent's queue\n2. Creates a new Result struct to track execution\n3. Executes the instruction through its action module\n4. Processes the execution result (directive, syscall or state)\n5. Returns the final Result struct","ref":"Jido.Runner.Simple.html#run/2-execution-process"},{"type":"function","title":"Parameters - Jido.Runner.Simple.run/2","doc":"* `agent` - The agent struct containing:\n    * `pending_instructions` - Queue of pending instructions\n    * `state` - Current agent state\n    * `id` - Agent identifier\n  * `opts` - Optional keyword list of execution options:\n    * Currently unused but reserved for future extensions","ref":"Jido.Runner.Simple.html#run/2-parameters"},{"type":"function","title":"Returns - Jido.Runner.Simple.run/2","doc":"* `{:ok, %Result{}}` - Successful execution with:\n    * `result_state` - Updated state map (for state results)\n    * `directives` - List of directives (for directive results)\n    * `syscalls` - List of syscalls (for syscall results)\n    * `status` - Set to :ok\n    * `error` - Set to nil\n  * `{:error, reason}` - Execution failed with:\n    * String error for queue empty condition\n    * Result struct with error details for execution failures","ref":"Jido.Runner.Simple.html#run/2-returns"},{"type":"function","title":"Examples - Jido.Runner.Simple.run/2","doc":"# Successful state update\n    {:ok, %Result{result_state: %{status: :complete}}} =\n      Runner.Simple.run(agent_with_state_update)\n\n    # Successful directive\n    {:ok, %Result{directives: [%EnqueueDirective{...}]}} =\n      Runner.Simple.run(agent_with_directive)\n\n    # Successful syscall\n    {:ok, %Result{syscalls: [%SpawnSyscall{...}]}} =\n      Runner.Simple.run(agent_with_syscall)\n\n    # Empty queue error\n    {:error, \"No pending instructions\"} =\n      Runner.Simple.run(agent_with_empty_queue)\n\n    # Execution error\n    {:error, %Result{error: error, status: :error}} =\n      Runner.Simple.run(agent_with_failing_action)","ref":"Jido.Runner.Simple.html#run/2-examples"},{"type":"function","title":"Error Handling - Jido.Runner.Simple.run/2","doc":"* Returns `{:error, \"No pending instructions\"}` for empty queue\n  * Returns `{:error, %Result{}}` with error details for execution failures\n  * All errors preserve the original agent state\n  * Failed executions do not affect the remaining queue","ref":"Jido.Runner.Simple.html#run/2-error-handling"},{"type":"function","title":"Logging - Jido.Runner.Simple.run/2","doc":"Debug logs are emitted at key points:\n  * Runner start with agent ID\n  * Instruction dequeue result\n  * Execution setup and workflow invocation\n  * Result processing and categorization","ref":"Jido.Runner.Simple.html#run/2-logging"},{"type":"type","title":"Jido.Runner.Simple.run_opts/0","doc":"","ref":"Jido.Runner.Simple.html#t:run_opts/0"},{"type":"type","title":"Jido.Runner.Simple.run_result/0","doc":"","ref":"Jido.Runner.Simple.html#t:run_result/0"},{"type":"module","title":"Jido.Error","doc":"Defines error structures and helper functions for Jido\n\nThis module provides a standardized way to create and handle errors within the Jido system.\nIt offers a set of predefined error types and functions to create, manipulate, and convert\nerror structures consistently across the application.\n\n> Why not use Exceptions?\n>\n> Jido is designed to be a functional system, strictly adhering to the use of result tuples.\n> This approach provides several benefits:\n>\n> 1. Consistent error handling: By using `{:ok, result}` or `{:error, reason}` tuples,\n>    we ensure a uniform way of handling success and failure cases throughout the system.\n>\n> 2. Composability: Monadic workflows can be easily chained together, allowing for\n>    cleaner and more maintainable code.\n>\n> 3. Explicit error paths: The use of result tuples makes error cases explicit,\n>    reducing the likelihood of unhandled errors.\n>\n> 4. No silent failures: Unlike exceptions, which can be silently caught and ignored,\n>    result tuples require explicit handling of both success and error cases.\n>\n> 5. Better testability: Monadic workflows are easier to test, as both success and\n>    error paths can be explicitly verified.\n>\n> By using this approach instead of exceptions, we gain more control over the flow of our\n> workflows and ensure that errors are handled consistently across the entire system.","ref":"Jido.Error.html"},{"type":"module","title":"Usage - Jido.Error","doc":"Use this module to create specific error types when exceptions occur in your Jido workflows.\nThis allows for consistent error handling and reporting throughout the system.\n\nExample:\n\n    defmodule MyWorkflow do\n      alias Jido.Error\n\n      def run(params) do\n        case validate(params) do\n          :ok -> perform_workflow(params)\n          {:error, reason} -> Error.validation_error(\"Invalid parameters\")\n        end\n      end\n    end","ref":"Jido.Error.html#module-usage"},{"type":"function","title":"Jido.Error.bad_request/3","doc":"Creates a new bad request error.\n\nUse this when the client sends an invalid or malformed request.","ref":"Jido.Error.html#bad_request/3"},{"type":"function","title":"Parameters - Jido.Error.bad_request/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#bad_request/3-parameters"},{"type":"function","title":"Example - Jido.Error.bad_request/3","doc":"iex> Jido.Error.bad_request(\"Missing required parameter 'user_id'\")\n    %Jido.Error{\n      type: :bad_request,\n      message: \"Missing required parameter 'user_id'\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#bad_request/3-example"},{"type":"function","title":"Jido.Error.capture_stacktrace/0","doc":"Captures the current stacktrace.\n\nThis function is useful when you want to capture the stacktrace at a specific point\nin your code, rather than at the point where the error is created. It drops the first\ntwo entries from the stacktrace to remove internal function calls related to this module.","ref":"Jido.Error.html#capture_stacktrace/0"},{"type":"function","title":"Returns - Jido.Error.capture_stacktrace/0","doc":"The current stacktrace as a list.","ref":"Jido.Error.html#capture_stacktrace/0-returns"},{"type":"function","title":"Example - Jido.Error.capture_stacktrace/0","doc":"iex> stacktrace = Jido.Error.capture_stacktrace()\n    iex> is_list(stacktrace)\n    true","ref":"Jido.Error.html#capture_stacktrace/0-example"},{"type":"function","title":"Jido.Error.compensation_error/3","doc":"Creates a new compensation error with details about the original error and compensation attempt.","ref":"Jido.Error.html#compensation_error/3"},{"type":"function","title":"Parameters - Jido.Error.compensation_error/3","doc":"- `original_error`: The error that triggered compensation\n- `details`: Optional map containing:\n  - `:compensated` - Boolean indicating if compensation succeeded\n  - `:compensation_result` - Result from successful compensation\n  - `:compensation_error` - Error from failed compensation\n- `stacktrace`: Optional stacktrace for debugging","ref":"Jido.Error.html#compensation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.compensation_error/3","doc":"iex> original_error = Jido.Error.execution_error(\"Failed to process payment\")\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: true,\n    ...>   compensation_result: %{refund_id: \"ref_123\"}\n    ...> })\n    %Jido.Error{\n      type: :compensation_error,\n      message: \"Compensation completed for: Failed to process payment\",\n      details: %{\n        compensated: true,\n        compensation_result: %{refund_id: \"ref_123\"},\n        original_error: %Jido.Error{...}\n      }\n    }\n\n    iex> # For failed compensation:\n    iex> Jido.Error.compensation_error(original_error, %{\n    ...>   compensated: false,\n    ...>   compensation_error: \"Refund failed\"\n    ...> })","ref":"Jido.Error.html#compensation_error/3-examples"},{"type":"function","title":"Jido.Error.config_error/3","doc":"Creates a new config error.\n\nUse this when there's an issue with the system or workflow configuration.","ref":"Jido.Error.html#config_error/3"},{"type":"function","title":"Parameters - Jido.Error.config_error/3","doc":"- `message`: A string describing the configuration error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#config_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.config_error/3","doc":"iex> Jido.Error.config_error(\"Invalid database connection string\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid database connection string\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#config_error/3-example"},{"type":"type","title":"Jido.Error.error_type/0","doc":"Defines the possible error types in the Jido system.\n\n- `:invalid_action`: Used when a action is improperly defined or used.\n- `:invalid_sensor`: Used when a sensor is improperly defined or used.\n- `:bad_request`: Indicates an invalid request from the client.\n- `:validation_error`: Used when input validation fails.\n- `:config_error`: Indicates a configuration issue.\n- `:execution_error`: Used when an error occurs during workflow execution.\n- `:workflow_error`: General workflow-related errors.\n- `:internal_server_error`: Indicates an unexpected internal error.\n- `:timeout`: Used when an workflow exceeds its time limit.\n- `:invalid_async_ref`: Indicates an invalid asynchronous workflow reference.\n- `:compensation_error`: Indicates an error occurred during compensation.\n- `:planning_error`: Used when an error occurs during workflow planning.","ref":"Jido.Error.html#t:error_type/0"},{"type":"function","title":"Jido.Error.execution_error/3","doc":"Creates a new execution error.\n\nUse this when an error occurs during the execution of an workflow.","ref":"Jido.Error.html#execution_error/3"},{"type":"function","title":"Parameters - Jido.Error.execution_error/3","doc":"- `message`: A string describing the execution error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#execution_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.execution_error/3","doc":"iex> Jido.Error.execution_error(\"Failed to process data\", %{step: \"data_transformation\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Failed to process data\",\n      details: %{step: \"data_transformation\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#execution_error/3-example"},{"type":"function","title":"Jido.Error.format_nimble_config_error/3","doc":"Formats a NimbleOptions validation error for configuration validation.\nUsed when validating configuration options at compile or runtime.","ref":"Jido.Error.html#format_nimble_config_error/3"},{"type":"function","title":"Parameters - Jido.Error.format_nimble_config_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Error.html#format_nimble_config_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.format_nimble_config_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:name], message: \"is required\"}\n    iex> Jido.Error.format_nimble_config_error(error, \"Action\")\n    \"Invalid configuration given to use Jido.Action for key [:name]: is required\"","ref":"Jido.Error.html#format_nimble_config_error/3-examples"},{"type":"function","title":"Jido.Error.format_nimble_validation_error/3","doc":"Formats a NimbleOptions validation error for parameter validation.\nUsed when validating runtime parameters.","ref":"Jido.Error.html#format_nimble_validation_error/3"},{"type":"function","title":"Parameters - Jido.Error.format_nimble_validation_error/3","doc":"- `error`: The NimbleOptions.ValidationError to format\n- `module_type`: String indicating the module type (e.g. \"Action\", \"Agent\", \"Sensor\")","ref":"Jido.Error.html#format_nimble_validation_error/3-parameters"},{"type":"function","title":"Examples - Jido.Error.format_nimble_validation_error/3","doc":"iex> error = %NimbleOptions.ValidationError{keys_path: [:input], message: \"is required\"}\n    iex> Jido.Error.format_nimble_validation_error(error, \"Action\")\n    \"Invalid parameters for Action at [:input]: is required\"","ref":"Jido.Error.html#format_nimble_validation_error/3-examples"},{"type":"function","title":"Jido.Error.internal_server_error/3","doc":"Creates a new internal server error.\n\nUse this for unexpected errors that occur within the system.","ref":"Jido.Error.html#internal_server_error/3"},{"type":"function","title":"Parameters - Jido.Error.internal_server_error/3","doc":"- `message`: A string describing the internal server error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#internal_server_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.internal_server_error/3","doc":"iex> Jido.Error.internal_server_error(\"Unexpected error in data processing\")\n    %Jido.Error{\n      type: :internal_server_error,\n      message: \"Unexpected error in data processing\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#internal_server_error/3-example"},{"type":"function","title":"Jido.Error.invalid_action/3","doc":"Creates a new invalid action error.\n\nUse this when a action is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_action/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_action/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_action/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_action/3","doc":"iex> Jido.Error.invalid_action(\"Action 'MyAction' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_action,\n      message: \"Action 'MyAction' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_action/3-example"},{"type":"function","title":"Jido.Error.invalid_async_ref/3","doc":"Creates a new invalid async ref error.\n\nUse this when an invalid reference to an asynchronous workflow is encountered.","ref":"Jido.Error.html#invalid_async_ref/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_async_ref/3","doc":"- `message`: A string describing the invalid async ref error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_async_ref/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_async_ref/3","doc":"iex> Jido.Error.invalid_async_ref(\"Invalid or expired async workflow reference\")\n    %Jido.Error{\n      type: :invalid_async_ref,\n      message: \"Invalid or expired async workflow reference\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_async_ref/3-example"},{"type":"function","title":"Jido.Error.invalid_sensor/3","doc":"Creates a new invalid sensor error.\n\nUse this when a sensor is improperly defined or used within the Jido system.","ref":"Jido.Error.html#invalid_sensor/3"},{"type":"function","title":"Parameters - Jido.Error.invalid_sensor/3","doc":"- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#invalid_sensor/3-parameters"},{"type":"function","title":"Example - Jido.Error.invalid_sensor/3","doc":"iex> Jido.Error.invalid_sensor(\"Sensor 'MySensor' is missing required callback\")\n    %Jido.Error{\n      type: :invalid_sensor,\n      message: \"Sensor 'MySensor' is missing required callback\",\n      details: nil,\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#invalid_sensor/3-example"},{"type":"function","title":"Jido.Error.new/4","doc":"Creates a new error struct with the given type and message.\n\nThis is a low-level function used by other error creation functions in this module.\nConsider using the specific error creation functions unless you need fine-grained control.","ref":"Jido.Error.html#new/4"},{"type":"function","title":"Parameters - Jido.Error.new/4","doc":"- `type`: The error type (see `t:error_type/0`).\n- `message`: A string describing the error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#new/4-parameters"},{"type":"function","title":"Examples - Jido.Error.new/4","doc":"iex> Jido.Error.new(:config_error, \"Invalid configuration\")\n    %Jido.Error{\n      type: :config_error,\n      message: \"Invalid configuration\",\n      details: nil,\n      stacktrace: [...]\n    }\n\n    iex> Jido.Error.new(:execution_error, \"Workflow failed\", %{step: \"data_processing\"})\n    %Jido.Error{\n      type: :execution_error,\n      message: \"Workflow failed\",\n      details: %{step: \"data_processing\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#new/4-examples"},{"type":"function","title":"Jido.Error.planning_error/3","doc":"Creates a new planning error.\n\nUse this when an error occurs during workflow planning.","ref":"Jido.Error.html#planning_error/3"},{"type":"function","title":"Parameters - Jido.Error.planning_error/3","doc":"- `message`: A string describing the planning error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#planning_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.planning_error/3","doc":"iex> Jido.Error.planning_error(\"Failed to plan workflow\", %{step: \"goal_analysis\"})\n    %Jido.Error{\n      type: :planning_error,\n      message: \"Failed to plan workflow\",\n      details: %{step: \"goal_analysis\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#planning_error/3-example"},{"type":"type","title":"Jido.Error.t/0","doc":"Represents a structured error in the Jido system.\n\nFields:\n- `type`: The category of the error (see `t:error_type/0`).\n- `message`: A human-readable description of the error.\n- `details`: Optional map containing additional error context.\n- `stacktrace`: Optional list representing the error's stacktrace.","ref":"Jido.Error.html#t:t/0"},{"type":"function","title":"Jido.Error.timeout/3","doc":"Creates a new timeout error.\n\nUse this when an workflow exceeds its allocated time limit.","ref":"Jido.Error.html#timeout/3"},{"type":"function","title":"Parameters - Jido.Error.timeout/3","doc":"- `message`: A string describing the timeout error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#timeout/3-parameters"},{"type":"function","title":"Example - Jido.Error.timeout/3","doc":"iex> Jido.Error.timeout(\"Workflow timed out after 30 seconds\", %{workflow: \"FetchUserData\"})\n    %Jido.Error{\n      type: :timeout,\n      message: \"Workflow timed out after 30 seconds\",\n      details: %{workflow: \"FetchUserData\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#timeout/3-example"},{"type":"function","title":"Jido.Error.to_map/1","doc":"Converts the error struct to a plain map.\n\nThis function transforms the error struct into a plain map,\nincluding the error type and stacktrace if available. It's useful\nfor serialization or when working with APIs that expect plain maps.","ref":"Jido.Error.html#to_map/1"},{"type":"function","title":"Parameters - Jido.Error.to_map/1","doc":"- `error`: An error struct of type `t:t/0`.","ref":"Jido.Error.html#to_map/1-parameters"},{"type":"function","title":"Returns - Jido.Error.to_map/1","doc":"A map representation of the error.","ref":"Jido.Error.html#to_map/1-returns"},{"type":"function","title":"Example - Jido.Error.to_map/1","doc":"iex> error = Jido.Error.validation_error(\"Invalid input\")\n    iex> Jido.Error.to_map(error)\n    %{\n      type: :validation_error,\n      message: \"Invalid input\",\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#to_map/1-example"},{"type":"function","title":"Jido.Error.validation_error/3","doc":"Creates a new validation error.\n\nUse this when input validation fails for an workflow.","ref":"Jido.Error.html#validation_error/3"},{"type":"function","title":"Parameters - Jido.Error.validation_error/3","doc":"- `message`: A string describing the validation error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#validation_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.validation_error/3","doc":"iex> Jido.Error.validation_error(\"Invalid email format\", %{field: \"email\", value: \"not-an-email\"})\n    %Jido.Error{\n      type: :validation_error,\n      message: \"Invalid email format\",\n      details: %{field: \"email\", value: \"not-an-email\"},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#validation_error/3-example"},{"type":"function","title":"Jido.Error.workflow_error/3","doc":"Creates a new workflow error.\n\nUse this for general workflow-related errors that don't fit into other categories.","ref":"Jido.Error.html#workflow_error/3"},{"type":"function","title":"Parameters - Jido.Error.workflow_error/3","doc":"- `message`: A string describing the workflow error.\n- `details`: (optional) A map containing additional error details.\n- `stacktrace`: (optional) The stacktrace at the point of error.","ref":"Jido.Error.html#workflow_error/3-parameters"},{"type":"function","title":"Example - Jido.Error.workflow_error/3","doc":"iex> Jido.Error.workflow_error(\"Workflow 'ProcessOrder' failed\", %{order_id: 12345})\n    %Jido.Error{\n      type: :workflow_error,\n      message: \"Workflow 'ProcessOrder' failed\",\n      details: %{order_id: 12345},\n      stacktrace: [...]\n    }","ref":"Jido.Error.html#workflow_error/3-example"},{"type":"module","title":"Jido.Util","doc":"A collection of utility functions for the Jido framework.\n\nThis module provides various helper functions that are used throughout the Jido framework,\nincluding:\n\n- ID generation\n- Name validation\n- Error handling\n- Logging utilities\n\nThese utilities are designed to support common operations and maintain consistency\nacross the Jido ecosystem. They encapsulate frequently used patterns and provide\na centralized location for shared functionality.\n\nMany of the functions in this module are used internally by other Jido modules,\nbut they can also be useful for developers building applications with Jido.","ref":"Jido.Util.html"},{"type":"function","title":"Jido.Util.generate_id/0","doc":"Generates a unique ID.","ref":"Jido.Util.html#generate_id/0"},{"type":"function","title":"Jido.Util.validate_actions/1","doc":"Validates that all modules in a list implement the Jido.Action behavior.\nUsed as a custom validator for NimbleOptions.\n\nThis function ensures that all provided modules are valid Jido.Action implementations\nby checking that they:\n1. Are valid Elixir modules that can be loaded\n2. Export the required __action_metadata__/0 function that indicates Jido.Action behavior","ref":"Jido.Util.html#validate_actions/1"},{"type":"function","title":"Parameters - Jido.Util.validate_actions/1","doc":"- `actions`: A list of module atoms or a single module atom to validate","ref":"Jido.Util.html#validate_actions/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_actions/1","doc":"- `{:ok, actions}` if all modules are valid Jido.Action implementations\n- `{:error, reason}` if any module is invalid","ref":"Jido.Util.html#validate_actions/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_actions/1","doc":"iex> defmodule ValidAction do\n    ...>   use Jido.Action,\n    ...>     name: \"valid_action\"\n    ...> end\n    ...> Jido.Util.validate_actions([ValidAction])\n    {:ok, [ValidAction]}\n\n    iex> Jido.Util.validate_actions([InvalidModule])\n    {:error, \"All actions must implement the Jido.Action behavior\"}\n\n    # Single module validation\n    iex> Jido.Util.validate_actions(ValidAction)\n    {:ok, [ValidAction]}","ref":"Jido.Util.html#validate_actions/1-examples"},{"type":"function","title":"Jido.Util.validate_name/1","doc":"Validates the name of a Action.\n\nThe name must contain only letters, numbers, and underscores.","ref":"Jido.Util.html#validate_name/1"},{"type":"function","title":"Parameters - Jido.Util.validate_name/1","doc":"- `name`: The name to validate.","ref":"Jido.Util.html#validate_name/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_name/1","doc":"- `{:ok, name}` if the name is valid.\n- `{:error, reason}` if the name is invalid.","ref":"Jido.Util.html#validate_name/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_name/1","doc":"iex> Jido.Action.validate_name(\"valid_name_123\")\n    {:ok, \"valid_name_123\"}\n\n    iex> Jido.Action.validate_name(\"invalid-name\")\n    {:error, \"The name must contain only letters, numbers, and underscores.\"}","ref":"Jido.Util.html#validate_name/1-examples"},{"type":"function","title":"Jido.Util.validate_runner/1","doc":"Validates that a module implements the Jido.Runner behavior.\n\nThis function ensures that the provided module is a valid Jido.Runner implementation\nby checking that it:\n1. Is a valid Elixir module that can be loaded\n2. Exports the required run/2 function that indicates Jido.Runner behavior","ref":"Jido.Util.html#validate_runner/1"},{"type":"function","title":"Parameters - Jido.Util.validate_runner/1","doc":"- `module`: The module atom to validate","ref":"Jido.Util.html#validate_runner/1-parameters"},{"type":"function","title":"Returns - Jido.Util.validate_runner/1","doc":"- `{:ok, module}` if the module is a valid Jido.Runner implementation\n- `{:error, Jido.Error.t()}` if the module is invalid","ref":"Jido.Util.html#validate_runner/1-returns"},{"type":"function","title":"Examples - Jido.Util.validate_runner/1","doc":"iex> defmodule ValidRunner do\n    ...>   @behaviour Jido.Runner\n    ...>   def run(agent, opts), do: {:ok, agent}\n    ...> end\n    ...> Jido.Util.validate_runner(ValidRunner)\n    {:ok, ValidRunner}\n\n    iex> Jido.Util.validate_runner(InvalidModule)\n    {:error, %Jido.Error{type: :validation_error, message: \"Module InvalidModule must implement run/2\"}}","ref":"Jido.Util.html#validate_runner/1-examples"},{"type":"extras","title":"Home","doc":"# Jido (自動)\n\n自動 (Jido) - A foundational framework for building autonomous, distributed agent systems in Elixir.\n\n> **Note**: Jido is a foundational framework that does not include AI or LLM capabilities out of the box. For AI integration, please see the separate [`jido_ai`](https://github.com/agentjido/jido_ai) package which provides custom actions for AI/LLM functionality like structured interactions with Anthropic's Claude models.\n\n[![Hex Version](https://img.shields.io/hexpm/v/jido.svg)](https://hex.pm/packages/jido)\n[![Hex Docs](http://img.shields.io/badge/hex.pm-docs-green.svg?style=flat)](https://hexdocs.pm/jido)\n[![Mix Test](https://github.com/agentjido/jido/actions/workflows/elixir-ci.yml/badge.svg)](https://github.com/agentjido/jido/actions/workflows/elixir-ci.yml)\n[![Apache 2 License](https://img.shields.io/hexpm/l/jido)](https://opensource.org/licenses/Apache-2.0)","ref":"readme.html"},{"type":"extras","title":"Overview - Home","doc":"Jido provides a robust foundation for building autonomous agents that can plan, execute, and adapt their behavior in distributed Elixir applications. Think of it as a toolkit for creating smart, composable workflows that can evolve and respond to their environment.\n\n> ⚠️ **Status**: Jido is under active development. The API is stable for Actions, Workflows, Agents and Sensors. We are actively working on the Agent Server and Supervisor for the 1.1 release.","ref":"readme.html#overview"},{"type":"extras","title":"Key Features - Home","doc":"- 🧩 **Composable Actions**: Build complex behaviors from simple, reusable actions\n- 🤖 **Autonomous Agents**: Self-directing entities that plan and execute workflows\n- 📡 **Real-time Sensors**: Event-driven data gathering and monitoring\n- 🔄 **Adaptive Learning**: Agents can modify their capabilities at runtime\n- 📊 **Built-in Telemetry**: Comprehensive observability and debugging\n- ⚡ **Distributed by Design**: Built for multi-node Elixir clusters\n- 🧪 **Testing Tools**: Rich helpers for unit and property-based testing","ref":"readme.html#key-features"},{"type":"extras","title":"Installation - Home","doc":"Add Jido to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:jido, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Core Concepts - Home","doc":"","ref":"readme.html#core-concepts"},{"type":"extras","title":"Actions - Home","doc":"Actions are the fundamental building blocks in Jido. Each Action is a discrete, reusable unit of work with a clear interface:\n\n```elixir\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats user data by trimming whitespace and normalizing email\",\n    schema: [\n      name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(params, _context) do\n    {:ok, %{\n      formatted_name: String.trim(params.name),\n      email: String.downcase(params.email)\n    }}\n  end\nend\n```\n\n[Learn more about Actions →](guides/actions.md)","ref":"readme.html#actions"},{"type":"extras","title":"Workflows - Home","doc":"Workflows chain Actions together to accomplish complex tasks. Jido handles data flow and error handling between steps:\n\n```elixir\nalias MyApp.Actions.{FormatUser, EnrichUserData, NotifyUser}\n\n{:ok, result} = Jido.Workflow.Chain.chain(\n  [FormatUser, EnrichUserData, NotifyUser],\n  %{\n    name: \"John Doe \",\n    email: \"JOHN@EXAMPLE.COM\"\n  }\n)\n```\n\n[Learn more about Workflows →](guides/actions.md#combining-actions-into-a-workflow)","ref":"readme.html#workflows"},{"type":"extras","title":"Agents - Home","doc":"Agents are stateful entities that can plan and execute Actions. They maintain their state through a schema and can adapt their behavior:\n\n```elixir\ndefmodule MyApp.CalculatorAgent do\n  use Jido.Agent,\n    name: \"calculator\",\n    description: \"An adaptive calculating agent\",\n    actions: [\n      MyApp.Actions.Add,\n      MyApp.Actions.Multiply,\n      Jido.Actions.Directives.RegisterAction\n    ],\n    schema: [\n      value: [type: :float, default: 0.0],\n      operations: [type: {:list, :atom}, default: []]\n    ]\n\n  def on_after_run(agent, result) do\n    # Track which operations we've used\n    ops = [result.action | agent.state.operations] |> Enum.uniq()\n    {:ok, %{agent | state: %{agent.state | operations: ops}}}\n  end\nend\n```\n\n[Learn more about Agents →](guides/agents.md)","ref":"readme.html#agents"},{"type":"extras","title":"Sensors - Home","doc":"Sensors provide real-time monitoring and data gathering for your agents:\n\n```elixir\ndefmodule MyApp.Sensors.OperationCounter do\n  use Jido.Sensor,\n    name: \"operation_counter\",\n    description: \"Tracks operation usage metrics\",\n    schema: [\n      emit_interval: [type: :pos_integer, default: 1000]\n    ]\n\n  def mount(opts) do\n    {:ok, Map.merge(opts, %{counts: %{}})}\n  end\n\n  def handle_info({:operation, name}, state) do\n    new_counts = Map.update(state.counts, name, 1, & &1 + 1)\n    {:noreply, %{state | counts: new_counts}}\n  end\nend\n```\n\n[Learn more about Sensors →](guides/sensors.md)","ref":"readme.html#sensors"},{"type":"extras","title":"Running in Production - Home","doc":"Start your agents under supervision:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Registry, keys: :unique, name: Jido.AgentRegistry},\n  {Phoenix.PubSub, name: MyApp.PubSub},\n  {Jido.Agent.Supervisor, pubsub: MyApp.PubSub},\n  {Jido.Agent.Server, \n    agent: MyApp.CalculatorAgent.new(),\n    name: \"calculator_1\"\n  }\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"readme.html#running-in-production"},{"type":"extras","title":"Example Use Cases - Home","doc":"- **Service Orchestration**: Coordinate complex workflows across multiple services\n- **Data Processing**: Build adaptive ETL pipelines that evolve with your data\n- **Business Automation**: Model complex business processes with autonomous agents\n- **System Monitoring**: Create smart monitoring agents that adapt to system behavior\n- **Transaction Management**: Handle multi-step transactions with built-in compensation\n- **Event Processing**: Process and react to event streams in real-time","ref":"readme.html#example-use-cases"},{"type":"extras","title":"Documentation - Home","doc":"- [📘 Getting Started Guide](guides/getting-started.md)\n- [🧩 Actions & Workflows](guides/actions.md)\n- [🤖 Building Agents](guides/agents.md)\n- [📡 Sensors & Monitoring](guides/sensors.md)\n- [🔄 Agent Directives](guides/directives.md)","ref":"readme.html#documentation"},{"type":"extras","title":"Contributing - Home","doc":"We welcome contributions! Here's how to get started:\n\n1. Fork the repository\n2. Run tests: `mix test`\n3. Run quality checks: `mix quality`\n4. Submit a PR\n\nPlease include tests for any new features or bug fixes.\n\nSee our [Contributing Guide](CONTRIBUTING.md) for detailed guidelines.","ref":"readme.html#contributing"},{"type":"extras","title":"Testing - Home","doc":"Jido is built with a test-driven mindset and provides comprehensive testing tools for building reliable agent systems. Our testing philosophy emphasizes:\n\n- Thorough test coverage for core functionality\n- Property-based testing for complex behaviors\n- Regression tests for every bug fix\n- Extensive testing helpers and utilities","ref":"readme.html#testing"},{"type":"extras","title":"Testing Utilities - Home","doc":"Jido provides several testing helpers:\n- `Jido.TestSupport` - Common testing utilities\n- Property-based testing via StreamData\n- Mocking support through Mimic\n- PubSub testing helpers\n- Signal assertion helpers","ref":"readme.html#testing-utilities"},{"type":"extras","title":"Running Tests - Home","doc":"```bash\n# Run the test suite\nmix test\n\n# Run with coverage reporting\nmix test --cover\n\n# Run the full quality check suite\nmix quality\n```\n\nWhile we strive for 100% test coverage, we prioritize meaningful tests that verify behavior over simple line coverage. Every new feature and bug fix includes corresponding tests to prevent regressions.","ref":"readme.html#running-tests"},{"type":"extras","title":"License - Home","doc":"Apache License 2.0 - See [LICENSE.md](LICENSE.md) for details.","ref":"readme.html#license"},{"type":"extras","title":"Support - Home","doc":"- 📚 [Documentation](https://hexdocs.pm/jido)\n- 💬 [GitHub Discussions](https://github.com/agentjido/jido/discussions)\n- 🐛 [Issue Tracker](https://github.com/agentjido/jido/issues)","ref":"readme.html#support"},{"type":"extras","title":"Getting Started","doc":"# Getting Started with Jido\n\nWelcome to **Jido** (自動), a powerful yet approachable Elixir framework for building autonomous, distributed agent systems. Whether you're building complex business processes, service orchestration, or adaptive workflows, Jido provides a robust foundation for your autonomous agents.","ref":"getting-started.html"},{"type":"extras","title":"Overview - Getting Started","doc":"Jido is built around four core concepts that work together to create intelligent, adaptable systems:\n\n1. **Actions**: Discrete, reusable units of work\n2. **Workflows**: Sequences of Actions that accomplish larger goals\n3. **Agents**: Stateful entities that can plan and execute workflows\n4. **Sensors**: Real-time monitoring and data gathering components\n\nLet's see how these pieces fit together.","ref":"getting-started.html#overview"},{"type":"extras","title":"Installation - Getting Started","doc":"Add Jido to your project's dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:jido, \"~> 1.0.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting-started.html#installation"},{"type":"extras","title":"Quick Start Example - Getting Started","doc":"Here's a complete example showing all major components working together in a user registration system:\n\n```elixir\n# 1. Define an Action\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats user data\",\n    schema: [\n      name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(params, _context) do\n    {:ok, %{\n      formatted_name: String.trim(params.name),\n      email: String.downcase(params.email)\n    }}\n  end\nend\n\n# 2. Create a Sensor to monitor registrations\ndefmodule MyApp.Sensors.RegistrationCounter do\n  use Jido.Sensor,\n    name: \"registration_counter\",\n    description: \"Tracks registration metrics\",\n    schema: [\n      emit_interval: [type: :pos_integer, default: 1000]\n    ]\n\n  def mount(opts) do\n    {:ok, Map.merge(opts, %{successful: 0, failed: 0})}\n  end\n\n  def handle_info({:registration, :success}, state) do\n    new_state = %{state | successful: state.successful + 1}\n    {:noreply, new_state}\n  end\nend\n\n# 3. Define an Agent to manage registrations\ndefmodule MyApp.Agents.RegistrationAgent do\n  use Jido.Agent,\n    name: \"registration_agent\",\n    description: \"Manages user registration process\",\n    actions: [MyApp.Actions.FormatUser],\n    schema: [\n      registrations: [type: :integer, default: 0],\n      last_registration: [type: {:or, [:map, nil]}, default: nil]\n    ]\n\n  def on_after_run(agent, result) do\n    new_count = agent.state.registrations + 1\n    {:ok, %{agent | state: %{agent.state | \n      registrations: new_count,\n      last_registration: result\n    }}}\n  end\nend\n\n# 4. Use it all together\ndefmodule MyApp.Example do\n  def register_user do\n    # Create and configure the agent\n    agent = MyApp.Agents.RegistrationAgent.new()\n    \n    # Execute a registration\n    {:ok, agent} = MyApp.Agents.RegistrationAgent.cmd(\n      agent,\n      MyApp.Actions.FormatUser,\n      %{\n        name: \"John Doe \",\n        email: \"JOHN@EXAMPLE.COM\"\n      }\n    )\n\n    # Results are stored in agent state\n    agent.state.last_registration\n  end\nend\n```","ref":"getting-started.html#quick-start-example"},{"type":"extras","title":"Key Concepts Explained - Getting Started","doc":"","ref":"getting-started.html#key-concepts-explained"},{"type":"extras","title":"Actions: The Building Blocks - Getting Started","doc":"Actions are small, focused pieces of functionality that:\n- Have a clear input schema\n- Perform one specific task\n- Return standardized results\n- Can be composed into workflows\n\n[Learn more about Actions →](guides/actions.md)","ref":"getting-started.html#actions-the-building-blocks"},{"type":"extras","title":"Workflows: Combining Actions - Getting Started","doc":"Workflows chain Actions together to accomplish larger goals:\n- Pass data between Actions automatically\n- Handle errors consistently\n- Support async execution\n- Enable conditional paths\n\nExample workflow:\n```elixir\nalias MyApp.Actions.{FormatUser, EnrichData, NotifyUser}\n\n{:ok, result} = Jido.Workflow.Chain.chain(\n  [FormatUser, EnrichData, NotifyUser],\n  %{name: \"John Doe\", email: \"john@example.com\"}\n)\n```","ref":"getting-started.html#workflows-combining-actions"},{"type":"extras","title":"Agents: Stateful Intelligence - Getting Started","doc":"Agents provide stateful wrappers around Actions with:\n- Schema-validated state management\n- Action planning and execution\n- Runtime adaptation through directives\n- Lifecycle callbacks\n\n[Learn more about Agents →](guides/agents.md)","ref":"getting-started.html#agents-stateful-intelligence"},{"type":"extras","title":"Sensors: Real-time Monitoring - Getting Started","doc":"Sensors gather data and monitor system state:\n- Run as independent processes\n- Emit standardized signals\n- Support real-time metrics\n- Enable adaptive behavior\n\n[Learn more about Sensors →](guides/sensors.md)","ref":"getting-started.html#sensors-real-time-monitoring"},{"type":"extras","title":"Running in Production - Getting Started","doc":"To run Jido in production, start your components under supervision:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Registry, keys: :unique, name: Jido.AgentRegistry},\n  {Phoenix.PubSub, name: MyApp.PubSub},\n  {Jido.Agent.Supervisor, pubsub: MyApp.PubSub},\n  \n  # Start your agent server\n  {Jido.Agent.Server, \n    agent: MyApp.Agents.RegistrationAgent.new(),\n    name: \"registration_1\"\n  },\n  \n  # Start your sensor\n  {MyApp.Sensors.RegistrationCounter,\n    name: \"counter_1\",\n    pubsub: MyApp.PubSub,\n    topic: \"registration_metrics\"\n  }\n]\n\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"getting-started.html#running-in-production"},{"type":"extras","title":"Next Steps - Getting Started","doc":"Now that you understand the basics, you can explore:\n\n1. [Building Complex Actions and Workflows](guides/actions.md)\n   - Action composition patterns\n   - Error handling\n   - Async execution\n   - Testing strategies\n\n2. [Creating Smart Agents](guides/agents.md)\n   - State management\n   - Planning and execution\n   - Lifecycle hooks\n   - Directive system\n\n3. [Implementing Sensors](guides/sensors.md)\n   - Signal design\n   - Real-time monitoring\n   - Metric collection\n   - Event processing\n\n4. [Using Agent Directives](guides/directives.md)\n   - Runtime adaptation\n   - Dynamic capabilities\n   - Self-modification patterns","ref":"getting-started.html#next-steps"},{"type":"extras","title":"Additional Resources - Getting Started","doc":"- [Complete Documentation](https://hexdocs.pm/jido)\n- [GitHub Repository](https://github.com/agentjido/jido)\n- [Examples and Tutorials](guides/)\n\nFor AI/LLM integration capabilities, see the separate [`jido_ai`](https://github.com/agentjido/jido_ai) package.","ref":"getting-started.html#additional-resources"},{"type":"extras","title":"Contributing - Getting Started","doc":"We welcome contributions! Please see our [Contributing Guide](CONTRIBUTING.md) for guidelines.","ref":"getting-started.html#contributing"},{"type":"extras","title":"License - Getting Started","doc":"Apache License 2.0 - See [LICENSE.md](LICENSE.md) for details.","ref":"getting-started.html#license"},{"type":"extras","title":"Actions & Workflows","doc":"# Actions & Workflows\n\nThis guide will introduce you to building composable, maintainable workflows using Jido's action-based architecture. Whether you're building complex business processes, integrating multiple services, or orchestrating distributed tasks, Jido provides a robust foundation for your workflows.","ref":"actions.html"},{"type":"extras","title":"Understanding Jido's Core Concepts - Actions & Workflows","doc":"Before we dive into code, let's understand why Jido exists and when you should use it.","ref":"actions.html#understanding-jido-s-core-concepts"},{"type":"extras","title":"Why Actions? - Actions & Workflows","doc":"You might be wondering: \"Why should I wrap my code in Actions when I could just write regular Elixir functions?\" This is an excellent question. The truth is, if you're building a standard Elixir application without agents, you probably shouldn't use Actions – regular Elixir modules and functions would be simpler and more direct.\n\nActions exist specifically to support agent systems. When you're building agents that need to make autonomous decisions about what steps to take, you need a way to package functionality into discrete, composable units that the agent can reason about and combine in novel ways. Think of Actions as LEGO bricks for agents – standardized, well-described building blocks that can be assembled in different combinations to solve problems.\n\nThe Action system provides several critical features for agent-based systems:\n- Standardized metadata that agents can use to understand what each Action does\n- Schema validation to ensure inputs are correct before execution\n- Consistent error handling and compensation patterns\n- Built-in telemetry and observability\n- Dynamic composition through workflows and chains\n\nIf you're not building agent systems, you should implement your logic directly in Elixir. But if you are working with agents, this foundation of composable Actions becomes essential for enabling autonomous behavior.\n\nNow, let's understand the key concepts that make up Jido:","ref":"actions.html#why-actions"},{"type":"extras","title":"Actions - Actions & Workflows","doc":"An Action is the fundamental building block in Jido. Think of an Action as a single, well-defined task that takes some input, performs a specific operation, and produces an output. Actions are designed to be small, focused, and composable – like UNIX commands that you can pipe together.\n\nFor example, an Action might:\n- Format and validate user input\n- Call an external API\n- Transform data\n- Send a notification","ref":"actions.html#actions"},{"type":"extras","title":"Workflows - Actions & Workflows","doc":"A Workflow is a sequence of Actions that work together to accomplish a larger goal. Workflows can be as simple as two Actions in sequence or as complex as dozens of Actions with conditional paths and error handling. The power of Jido comes from being able to compose simple Actions into sophisticated Workflows.","ref":"actions.html#workflows"},{"type":"extras","title":"Chains - Actions & Workflows","doc":"A Chain is how we connect Actions together into a Workflow. When you chain Actions, the output of each Action becomes available to the next Action in the chain. Think of it like a pipeline where data flows through each step, getting enriched or transformed along the way.\n\nNow, let's see how to build these concepts in practice.","ref":"actions.html#chains"},{"type":"extras","title":"Creating Your First Action - Actions & Workflows","doc":"Let's create a simple Action that formats user data. This example will help you understand the basic structure of an Action and how Jido handles data transformation.\n\n```elixir\ndefmodule MyApp.Actions.FormatUser do\n  use Jido.Action,\n    name: \"format_user\",\n    description: \"Formats user data by trimming whitespace and normalizing email\",\n    schema: [\n      name: [\n        type: :string, \n        required: true,\n        doc: \"User's full name - will be trimmed\"\n      ],\n      email: [\n        type: :string,\n        required: true,\n        doc: \"Email address - will be converted to lowercase\"\n      ],\n      age: [\n        type: :integer,\n        required: true,\n        doc: \"User's age in years\"\n      ]\n    ]\n\n  @impl true\n  def run(params, _context) do\n    %{name: name, email: email, age: age} = params\n\n    {:ok, %{\n      formatted_name: String.trim(name),\n      email: String.downcase(email),\n      age: age,\n      is_adult: age >= 18\n    }}\n  end\nend\n```\n\nLet's break down what's happening here:\n\n1. We define our Action as a module using `use Jido.Action`\n2. We provide metadata about our Action:\n   - `name`: A unique identifier for the Action\n   - `description`: What the Action does\n   - `schema`: The expected input parameters and their types\n3. We implement the `run/2` callback which contains our Action's logic\n4. We return a tuple `{:ok, result}` with our transformed data\n\nThe schema is particularly important – it's how Jido validates input before running your Action. If someone tries to run this Action with invalid input (like a missing email or an age that's not a number), Jido will return an error before your code even runs.","ref":"actions.html#creating-your-first-action"},{"type":"extras","title":"Running Your First Action - Actions & Workflows","doc":"There are two ways to run an Action. Let's understand both approaches:","ref":"actions.html#running-your-first-action"},{"type":"extras","title":"1. Direct Execution - Actions & Workflows","doc":"The simplest way to run an Action is to call it directly:\n\n```elixir\n# Notice the trailing space in name and uppercase email - our Action will clean these\n{:ok, result} = FormatUser.run(%{\n  name: \"John Doe \",          \n  email: \"JOHN@EXAMPLE.COM\",  \n  age: 30\n})\n```\n\nThis will give us:\n```elixir\n%{\n  formatted_name: \"John Doe\",    # Space removed\n  email: \"john@example.com\",     # Converted to lowercase\n  age: 30,\n  is_adult: true                 # Derived from age\n}\n```\n\nDirect execution is great for testing and development, but it doesn't give you all of Jido's runtime features.","ref":"actions.html#1-direct-execution"},{"type":"extras","title":"2. Using the Workflow Runtime - Actions & Workflows","doc":"For production use, you'll want to use Jido's Workflow runtime:\n\n```elixir\n{:ok, result} = Jido.Workflow.run(FormatUser, %{\n  name: \"John Doe\",\n  email: \"john@example.com\",\n  age: 30\n})\n```\n\nThe Workflow runtime provides several benefits:\n- Telemetry events for monitoring\n- Consistent error handling\n- Timeout management\n- Retries and circuit breaking\n- Context passing","ref":"actions.html#2-using-the-workflow-runtime"},{"type":"extras","title":"Building More Complex Actions - Actions & Workflows","doc":"Now that we understand basic Actions, let's create two more that we'll use to build a complete user registration workflow:\n\n```elixir\ndefmodule MyApp.Actions.EnrichUserData do\n  use Jido.Action,\n    name: \"enrich_user_data\",\n    description: \"Adds username and avatar URL to user data\",\n    schema: [\n      formatted_name: [type: :string, required: true],\n      email: [type: :string, required: true]\n    ]\n\n  def run(%{formatted_name: name, email: email}, _context) do\n    {:ok, %{\n      username: generate_username(name),\n      avatar_url: get_gravatar_url(email)\n    }}\n  end\n\n  defp generate_username(name) do\n    name\n    |> String.downcase()\n    |> String.replace(\" \", \".\")\n  end\n\n  defp get_gravatar_url(email) do\n    hash = :crypto.hash(:md5, email) |> Base.encode16(case: :lower)\n    \"https://www.gravatar.com/avatar/#{hash}\"\n  end\nend\n\ndefmodule MyApp.Actions.NotifyUser do\n  use Jido.Action,\n    name: \"notify_user\",\n    description: \"Sends welcome notification to user\",\n    schema: [\n      email: [type: :string, required: true],\n      username: [type: :string, required: true]\n    ]\n\n  def run(%{email: email, username: username}, _context) do\n    # In a real app, you'd send an actual email\n    Logger.info(\"Sending welcome email to #{email}\")\n    \n    {:ok, %{\n      notification_sent: true,\n      notification_type: \"welcome_email\",\n      recipient: %{\n        email: email,\n        username: username\n      }\n    }}\n  end\nend\n```\n\nEach Action has a single responsibility and clearly defined inputs and outputs. This modularity makes our code easier to test and maintain.","ref":"actions.html#building-more-complex-actions"},{"type":"extras","title":"Chaining Actions Together - Actions & Workflows","doc":"Now comes the powerful part – we can chain these Actions together to create a complete user registration workflow. When we chain Actions, the output of each Action is automatically merged with the existing data and passed to the next Action.\n\n```elixir\n{:ok, result} = Chain.chain(\n  [\n    FormatUser,\n    EnrichUserData,\n    NotifyUser\n  ],\n  %{\n    name: \"John Doe\",\n    email: \"john@example.com\",\n    age: 30\n  }\n)\n```\n\nHere's what happens in this chain:\n\n1. `FormatUser` runs first:\n   - Takes the raw input\n   - Formats the name and email\n   - Adds the is_adult flag\n\n2. `EnrichUserData` runs next:\n   - Gets the formatted name and email from previous step\n   - Adds username and avatar URL\n   - Results are merged with existing data\n\n3. `NotifyUser` runs last:\n   - Uses the email and username from previous steps\n   - Sends the notification\n   - Adds notification status to results\n\nThe final result contains all the accumulated data:\n```elixir\n%{\n  formatted_name: \"John Doe\",\n  email: \"john@example.com\",\n  age: 30,\n  is_adult: true,\n  username: \"john.doe\",\n  avatar_url: \"https://www.gravatar.com/avatar/...\",\n  notification_sent: true,\n  notification_type: \"welcome_email\",\n  recipient: %{\n    email: \"john@example.com\",\n    username: \"john.doe\"\n  }\n}\n```","ref":"actions.html#chaining-actions-together"},{"type":"extras","title":"Advanced Chain Features - Actions & Workflows","doc":"As your workflows grow more complex, Jido provides several advanced features:","ref":"actions.html#advanced-chain-features"},{"type":"extras","title":"Passing Context - Actions & Workflows","doc":"Context lets you provide additional data that any Action in the chain can access:\n\n```elixir\nChain.chain(\n  [FormatUser, EnrichUserData, NotifyUser],\n  user_data,\n  context: %{\n    tenant_id: \"123\",\n    environment: \"test\"\n  }\n)\n```\n\nThis is useful for passing configuration, authentication info, or other cross-cutting concerns.","ref":"actions.html#passing-context"},{"type":"extras","title":"Overriding Parameters - Actions & Workflows","doc":"You can override specific parameters for individual Actions in the chain:\n\n```elixir\nChain.chain(\n  [\n    {FormatUser, [name: \"Jane Doe\"]},  # Override name just for this Action\n    EnrichUserData,\n    NotifyUser\n  ],\n  user_data\n)\n```\n\nThis is particularly useful when you need to customize behavior for specific cases.","ref":"actions.html#overriding-parameters"},{"type":"extras","title":"Async Execution - Actions & Workflows","doc":"For long-running workflows, you can run Actions asynchronously:\n\n```elixir\n# Start the workflow\nasync_ref = Jido.Workflow.run_async(FormatUser, user_data)\n\n# Do other work...\n\n# Get the result when you need it\n{:ok, result} = Jido.Workflow.await(async_ref)\n```","ref":"actions.html#async-execution"},{"type":"extras","title":"Testing Your Actions - Actions & Workflows","doc":"Testing is a crucial part of working with Jido. Here's how to test your Actions thoroughly:\n\n```elixir\ndefmodule MyApp.Actions.UserRegistrationTest do\n  use ExUnit.Case, async: true\n  \n  @valid_user_data %{\n    name: \"John Doe \",\n    email: \"JOHN@EXAMPLE.COM\",\n    age: 30\n  }\n  \n  describe \"individual action tests\" do\n    test \"FormatUser formats and validates user data\" do\n      {:ok, result} = FormatUser.run(@valid_user_data, %{})\n      \n      assert result.formatted_name == \"John Doe\"\n      assert result.email == \"john@example.com\"\n      assert result.is_adult == true\n    end\n  end\n  \n  describe \"chaining actions\" do\n    test \"chains all user registration actions together\" do\n      {:ok, result} = Chain.chain(\n        [FormatUser, EnrichUserData, NotifyUser],\n        @valid_user_data\n      )\n      \n      assert result.formatted_name == \"John Doe\"\n      assert result.username == \"john.doe\"\n      assert result.notification_sent == true\n    end\n    \n    test \"chain stops on first error\" do\n      invalid_data = %{@valid_user_data | email: nil}\n      \n      {:error, error} = Chain.chain(\n        [FormatUser, EnrichUserData, NotifyUser],\n        invalid_data\n      )\n      \n      assert error.type == :validation_error\n    end\n  end\nend\n```\n\nThe complete test suite in `test/workflow/user_registration_test.exs` shows more examples, including:\n- Testing async workflows\n- Testing with context\n- Testing parameter overrides\n- Testing error conditions","ref":"actions.html#testing-your-actions"},{"type":"extras","title":"Best Practices - Actions & Workflows","doc":"As you build with Jido, keep these principles in mind:\n\n1. **Single Responsibility**: Each Action should do one thing well. If an Action is doing too much, split it into smaller Actions.\n\n2. **Clear Contracts**: Use schemas to define clear input requirements. Document what each parameter is for.\n\n3. **Error Handling**: Return clear error messages. Use the Error structs provided by Jido.\n\n4. **Testing**: Test Actions both individually and in chains. Test happy paths and error cases.\n\n5. **Stateless Design**: Actions should be stateless and idempotent when possible. Use context for state that needs to be shared.\n\n6. **Documentation**: Document your Actions, especially any side effects or external dependencies.","ref":"actions.html#best-practices"},{"type":"extras","title":"Next Steps - Actions & Workflows","doc":"Now that you understand the basics, you can explore:\n\n- More complex workflow patterns\n- Error compensation and rollbacks\n- Integration with external services\n- Custom Action behaviors\n- Telemetry and monitoring\n\nThe complete test suite provides many more examples and patterns to learn from.\n\nHappy building with Jido!","ref":"actions.html#next-steps"},{"type":"extras","title":"Agents","doc":"# Agents\n\nIn our previous guide, we explored how Actions serve as composable building blocks for workflows. Now, let's discover how Agents bring these Actions to life by providing a stateful, intelligent wrapper around them.","ref":"agents.html"},{"type":"extras","title":"What are Agents? - Agents","doc":"At their core, Jido Agents are simply data structures that understand how to maintain and transform their own state. While you might be familiar with Elixir's built-in Agent processes for state management, Jido Agents take a layered approach. The `Jido.Agent` module provides the foundational data structure, and the `Jido.Agent.Server` provides the GenServer implementation that manages the Agent's runtime state within OTP.\n\nThink of a Jido Agent as a specialized map that knows two important things:\n1. What shape its data should have (through its schema)\n2. What transformations it can apply to that data (through its registered Actions)\n\nWhen you create an Agent, you're not starting a process or service - you're creating a data structure that can validate its own contents and understands how to transform itself through Actions. This pure data approach means Agents are predictable, testable, and can be easily serialized or persisted.","ref":"agents.html#what-are-agents"},{"type":"extras","title":"Agents as Data Transformation Engines - Agents","doc":"The fundamental job of an Agent is to transform data in controlled, predictable ways. It does this through a simple pattern:\n\n1. It holds some initial state (input data)\n2. It applies one or more Actions to transform that data\n3. It either returns the transformed data or updates its own state\n\nFor example, imagine an Agent processing user registration data. It might:\n1. Start with raw input: `%{name: \"JOHN DOE \", email: \"JOHN@EXAMPLE.COM\"}`\n2. Apply a formatting Action that normalizes the data\n3. Store or return the result: `%{name: \"John Doe\", email: \"john@example.com\"}`","ref":"agents.html#agents-as-data-transformation-engines"},{"type":"extras","title":"The Role of State - Agents","doc":"An Agent's state is just a schema-validated map. The schema serves several purposes:\n- It documents what fields the Agent can contain\n- It ensures data consistency through type validation\n- It provides default values for fields\n- It makes the Agent's purpose clear to other developers\n\nThe schema isn't just for validation - it's the Agent's contract with the rest of your system about what data it manages and how that data should look.\n\n#### State Validation Modes\n\nBy default, Agents use a permissive validation approach - they validate fields defined in the schema while allowing additional unknown fields. This flexibility supports development workflows and experimental features. For example, you might want to temporarily store debugging information or track experimental metrics without modifying your schema.\n\nHowever, when you need stricter guarantees about your data, you can enable strict validation mode when setting state. In strict mode, the Agent will reject any fields not defined in its schema. This is valuable when:\n\n- You need to guarantee complete state consistency\n- You want to catch typos or mistakes in field names early\n- You're working with sensitive data where extra fields could be problematic\n- You want to maintain a strict contract about what data an Agent manages\n\nYou control this behavior through the `strict_validation` option when setting state, which we'll see in practice shortly.","ref":"agents.html#the-role-of-state"},{"type":"extras","title":"Actions and Transformation - Agents","doc":"While the Agent holds state, Actions do the actual work of transformation. The Agent's job is to:\n- Validate that an Action is registered and can be used\n- Provide its current state as input to the Action\n- Validate and store (or return) the Action's output\n\nThink of it like a pipeline where data flows through one or more transformations, with the Agent ensuring each step maintains data integrity.","ref":"agents.html#actions-and-transformation"},{"type":"extras","title":"Planning and Execution - Agents","doc":"One key feature of Agents is their ability to plan sequences of transformations before executing them. When you plan Actions on an Agent, you're creating a queue of transformations to apply. This separation between planning and execution lets you:\n\n1. Validate the entire transformation sequence before running it\n2. Ensure Actions will receive the right input at each step\n3. Choose whether to apply results to Agent state or just return them\n4. Control how results flow between Actions in the sequence\n\nThe actual execution happens through a Runner, which orchestrates how the transformations occur and how results flow between steps.","ref":"agents.html#planning-and-execution"},{"type":"extras","title":"Creating Your First Agent - Agents","doc":"Let's create a simple User Registration Agent that will help us understand these concepts. We'll start with the same user registration workflow from our Actions guide, but now we'll wrap it in an Agent:\n\n```elixir\ndefmodule MyApp.UserAgent do\n  use Jido.Agent,\n    name: \"user_agent\",\n    description: \"Manages user registration\",\n    # Actions this agent can use\n    actions: [FormatUser, EnrichUserData, NotifyUser],\n    # State schema\n    schema: [\n      # Input fields\n      name: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"User's raw input name\"\n      ],\n      email: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"User's raw input email\"\n      ],\n      age: [\n        type: {:or, [:integer, nil]}, \n        default: nil,\n        doc: \"User's age in years\"\n      ],\n      \n      # Fields that will store action results\n      formatted_name: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"Name after formatting\"\n      ],\n      username: [\n        type: {:or, [:string, nil]}, \n        default: nil,\n        doc: \"Generated username\"\n      ],\n      notification_sent: [\n        type: :boolean, \n        default: false,\n        doc: \"Whether welcome notification was sent\"\n      ]\n    ]\nend\n```\n\nLet's break down this definition:\n\n1. The `name` and `description` help other parts of the system understand what this Agent does\n2. The `actions` list declares which Actions this Agent can use\n3. The `schema` defines what information this Agent can remember, including both input fields and expected action results","ref":"agents.html#creating-your-first-agent"},{"type":"extras","title":"Working with Agents - Agents","doc":"Now that we have our Agent defined, let's see how to use it. Working with Agents follows a natural progression of create → set → plan → run.","ref":"agents.html#working-with-agents"},{"type":"extras","title":"Creating an Agent - Agents","doc":"First, we create a new instance of our Agent:\n\n```elixir\n# Create a new agent\nagent = MyApp.UserAgent.new()\n\n# It starts with default values\nagent.state.name         #=> nil\nagent.state.email       #=> nil\nagent.state.username    #=> nil\n\n# Each agent has a unique ID\nagent.id  #=> \"ag_123...\"\n```","ref":"agents.html#creating-an-agent"},{"type":"extras","title":"Setting State - Agents","doc":"Before we can process anything, we need to give our Agent some data to work with. We can use `set/2` in either permissive or strict mode:\n\n```elixir\n# Default permissive mode allows unknown fields\n{:ok, agent} = MyApp.UserAgent.set(agent, %{\n  name: \"John Doe\",\n  email: \"john@example.com\",\n  debug_info: %{source: \"test\"}  # Not in schema, but allowed\n})\n\n# Strict mode rejects unknown fields\n{:error, error} = MyApp.UserAgent.set(agent, \n  %{\n    name: \"John Doe\",\n    unknown_field: true\n  },\n  strict_validation: true\n)\n\n# The error clearly identifies rejected fields\nassert error.message =~ \"Unknown fields: [:unknown_field]\"\n```\n\nState updates follow these rules:\n- Each update returns a new immutable copy of the agent\n- Schema fields are always validated\n- Unknown fields are allowed by default but can be rejected with strict validation\n- Multiple fields can be updated at once\n- Existing fields are preserved unless explicitly changed","ref":"agents.html#setting-state"},{"type":"extras","title":"Planning Actions - Agents","doc":"Once our Agent has some state, we can plan what Actions it should take:\n\n```elixir\n# Plan a single action using current state\n{:ok, agent} = MyApp.UserAgent.plan(agent, FormatUser)\n\n# Plan multiple actions with explicit parameters\n{:ok, agent} = MyApp.UserAgent.plan(agent, [\n  {FormatUser, agent.state},\n  EnrichUserData,\n  NotifyUser\n])\n\n# Planning doesn't execute - state remains unchanged\nassert agent.state.formatted_name == nil\n```\n\nThe planning phase lets us:\n- Validate that all actions are registered\n- Set up the transformation sequence\n- Prepare parameters for each step\n- Check for obvious problems before execution\n\n> Why doesn't the agent pass it's state to the first action?\n>\n> Great question - and there are two reasons for this:\n> 1. The Runner is the component responsible for passing state to the Action - so in our contrived example, it's important to demonstrate that the state is being explicitly passed\n> 2. The Agent has sensible defaults that try to remove as many assumptions and require explicit programming.  Passing the state by default is a good example of this.  If you want to pass the state to the first action, you can do so explicitly.","ref":"agents.html#planning-actions"},{"type":"extras","title":"Running Actions - Agents","doc":"Once we've planned our Actions, we can execute them:\n\n```elixir\n# Run and apply results to state\n{:ok, agent} = MyApp.UserAgent.run(agent, \n  apply_state: true,\n  runner: Jido.Runner.Chain\n)\n\n# Verify transformations happened\nassert agent.state.formatted_name == \"John Doe\"\nassert agent.state.username == \"john.doe\"\n```\n\nOr we can run without updating state to inspect the results first:\n\n```elixir\n# Run without applying state changes\n{:ok, agent} = MyApp.UserAgent.run(agent, apply_state: false)\n\n# State unchanged but results available\nassert agent.state.formatted_name == nil\nassert agent.result.result_state.formatted_name == \"John Doe\"\n```\n\nThis separation between execution and state updates gives us control over when and how our Agent's state changes.","ref":"agents.html#running-actions"},{"type":"extras","title":"Putting It All Together: Commands - Agents","doc":"While we can use `set`, `plan`, and `run` separately, Jido provides a convenient `cmd/4` function that combines them:\n\n```elixir\n{:ok, agent} = MyApp.UserAgent.cmd(\n  agent,\n  [{FormatUser, agent.state}, EnrichUserData],  # Actions to run\n  %{age: 30},                                   # State to set\n  apply_state: true,                           # Update state with results\n  runner: Jido.Runner.Chain                    # Use chain runner\n)\n\n# Everything happened in one step\nassert agent.state.formatted_name == \"John Doe\"\nassert agent.state.username == \"john.doe\"\nassert agent.state.age == 30\n```","ref":"agents.html#putting-it-all-together-commands"},{"type":"extras","title":"Best Practices - Agents","doc":"As you build with Agents, keep these principles in mind:\n\n1. Use strict validation when data consistency is critical and permissive validation during development or for temporary data.\n\n2. Keep your schema focused on the data your Agent truly needs to manage. Just because you can store additional fields doesn't mean you should.\n\n3. Let your Actions handle complex transformations and use `set/2` primarily for direct state updates.\n\n4. Take advantage of the separation between planning and execution to validate operations before running them.\n\n5. Consider whether to apply results to state based on your use case - sometimes you want to inspect results before committing them.","ref":"agents.html#best-practices"},{"type":"extras","title":"Next Steps - Agents","doc":"Now that you understand the basics of Agents, you can explore:\n- Complex action chains\n- Conditional execution paths\n- Error handling and recovery\n- State persistence\n- Agent supervision and lifecycle\n- Inter-agent communication\n\nThe test suite provides many examples of these advanced patterns. Remember: Agents are most powerful when they maintain focused state and use well-defined Actions to transform that state. Keep your Agents focused, their state clean, and their Actions clear.","ref":"agents.html#next-steps"},{"type":"extras","title":"Sensors","doc":"# Signals and Sensors\n\nIn our previous guides, we explored how Actions serve as composable building blocks and how Agents provide stateful wrappers around them. Now, let's discover how Signals and Sensors enable real-time event monitoring and metrics collection in your Jido system.","ref":"sensors.html"},{"type":"extras","title":"Understanding Signals and Sensors - Sensors","doc":"Before we dive into code, let's understand what Signals and Sensors are and why they're important in a distributed system.","ref":"sensors.html#understanding-signals-and-sensors"},{"type":"extras","title":"What are Signals? - Sensors","doc":"At their core, Signals are standardized event messages that flow through your Jido system. They implement the CloudEvents specification (v1.0.2) with Jido-specific extensions, providing a consistent format for all system events. Think of Signals as the nervous system of your application - they carry information about:\n\n- State changes\n- Metric updates\n- System events\n- Process lifecycle events\n- Command execution results\n\nEvery Signal includes standard CloudEvents fields like:\n- `id`: A unique identifier\n- `source`: Where the signal originated\n- `type`: What kind of event it represents\n- `subject`: What entity the signal relates to\n- `data`: The actual event payload\n\nPlus Jido-specific fields for:\n- `jidoaction`: Action definitions for command signals\n- `jidoopts`: Options for signal processing","ref":"sensors.html#what-are-signals"},{"type":"extras","title":"Why Sensors? - Sensors","doc":"Sensors are independent GenServer processes that gather external information and update agent state independently. This architectural choice is crucial for distributed systems where a single agent might have dozens of sensors feeding it information simultaneously - from API monitoring and metrics tracking to event stream processing.\n\nRunning sensors as standalone processes under OTP supervision provides key benefits:\n- **Fault Isolation**: A failing sensor won't crash the agent or other sensors\n- **Independent Scaling**: Sensors can be distributed across nodes based on load\n- **Resource Management**: Each sensor manages its own memory and process queue\n- **Dynamic Operation**: Sensors can be started, stopped, and supervised independently\n\nWhile this guide covers defining and starting individual sensors, later guides will explain how agents dynamically manage multiple sensors through registration, lifecycle management, and event routing.\n\nSensors serve several purposes:\n- Track metrics and state changes in real-time\n- Process and aggregate event streams\n- Generate alerts for important conditions\n- Maintain recent event history\n\nA key role of Sensors is translating external events into standardized Signals that agents can understand and consume. For example, a sensor might:\n- Listen for HTTP webhook events and emit corresponding agent-ready signals\n- Watch a message queue and transform messages into signals\n- Monitor file changes and generate file-event signals\n- Subscribe to external event streams and normalize them into signals\n\nThink of sensors as specialized translators that convert various external events into a consistent Signal format that agents know how to process, all while operating independently under OTP supervision.","ref":"sensors.html#why-sensors"},{"type":"extras","title":"Creating Your First Sensor - Sensors","doc":"Let's build on our user registration example by creating a Sensor that monitors registration success rates. This will help us understand usage patterns and identify potential issues.\n\n```elixir\ndefmodule MyApp.Sensors.RegistrationCounter do\n  @moduledoc \"\"\"\n  Tracks user registration success and failure metrics.\n  \"\"\"\n  use Jido.Sensor,\n    name: \"registration_counter\",\n    description: \"Monitors registration successes and failures\",\n    category: :metrics,\n    tags: [:registration, :counter],\n    vsn: \"1.0.0\",\n    schema: [\n      emit_interval: [\n        type: :pos_integer,\n        default: 1000,\n        doc: \"Interval between metric emissions in ms\"\n      ]\n    ]\n\n  def mount(opts) do\n    state = Map.merge(opts, %{\n      successful: 0,\n      failed: 0\n    })\n    \n    schedule_emit(state)\n    {:ok, state}\n  end\n\n  def generate_signal(state) do\n    total = state.successful + state.failed\n    success_rate = if total > 0, do: state.successful / total * 100, else: 0\n\n    Jido.Signal.new(%{\n      source: \"#{state.sensor.name}:#{state.id}\",\n      subject: \"registration_counts\",\n      type: \"registration.metrics\",\n      data: %{\n        successful: state.successful,\n        failed: state.failed,\n        total: total,\n        success_rate: success_rate\n      }\n    })\n  end\n\n  def handle_info(:emit, state) do\n    with {:ok, signal} <- generate_signal(state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      schedule_emit(state)\n      {:noreply, state}\n    else\n      error ->\n        Logger.warning(\"Error generating/publishing signal: #{inspect(error)}\")\n        schedule_emit(state)\n        {:noreply, state}\n    end\n  end\n\n  def handle_info({:registration, :success}, state) do\n    new_state = %{state | successful: state.successful + 1}\n    \n    with {:ok, signal} <- generate_signal(new_state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      {:noreply, new_state}\n    else\n      error ->\n        Logger.warning(\"Error broadcasting success signal: #{inspect(error)}\")\n        {:noreply, new_state}\n    end\n  end\n\n  def handle_info({:registration, :failure}, state) do\n    new_state = %{state | failed: state.failed + 1}\n    \n    with {:ok, signal} <- generate_signal(new_state),\n         :ok <- Phoenix.PubSub.broadcast(state.pubsub, state.topic, {:sensor_signal, signal}) do\n      {:noreply, new_state}\n    else\n      error ->\n        Logger.warning(\"Error broadcasting failure signal: #{inspect(error)}\")\n        {:noreply, new_state}\n    end\n  end\n\n  defp schedule_emit(state) do\n    Process.send_after(self(), :emit, state.emit_interval)\n  end\nend\n```\n\nLet's break down what's happening in this Sensor:\n\n1. We use `use Jido.Sensor` to define our Sensor, providing metadata like:\n   - `name`: Unique identifier\n   - `description`: What the Sensor monitors\n   - `category`: Classification for grouping\n   - `tags`: Labels for filtering\n   - `schema`: Configuration options\n\n2. The `mount/1` callback initializes our Sensor's state with:\n   - Counters for successes and failures\n   - Configuration from schema\n   - Scheduled metric emissions\n\n3. `generate_signal/1` creates standardized Signal structs containing:\n   - Current metric values\n   - Calculated success rate\n   - Source identification\n   - Event type and subject\n\n4. Message handlers track events and emit signals:\n   - `:emit` for regular metric updates\n   - `{:registration, :success}` for successful registrations\n   - `{:registration, :failure}` for failed registrations","ref":"sensors.html#creating-your-first-sensor"},{"type":"extras","title":"Working with Signals - Sensors","doc":"Signals provide a standardized way to represent events in your system. Let's look at how to create and handle them:","ref":"sensors.html#working-with-signals"},{"type":"extras","title":"Creating Signals - Sensors","doc":"The simplest way to create a Signal is using `Jido.Signal.new/1`:\n\n```elixir\n{:ok, signal} = Jido.Signal.new(%{\n  source: \"my_component\",\n  type: \"user.registered\",\n  subject: \"user_123\",\n  data: %{\n    username: \"john.doe\",\n    timestamp: DateTime.utc_now()\n  }\n})\n```\n\nSignals are validated to ensure they contain required fields and follow the CloudEvents spec.","ref":"sensors.html#creating-signals"},{"type":"extras","title":"Publishing Signals - Sensors","doc":"Signals are typically published through Phoenix.PubSub:\n\n```elixir\nPhoenix.PubSub.broadcast(pubsub, topic, signal)\n```\n\nThe signal will be received directly by any subscribers to that topic.","ref":"sensors.html#publishing-signals"},{"type":"extras","title":"Subscribing to Signals - Sensors","doc":"To receive Signals, subscribe to the relevant PubSub topic:\n\n```elixir\nPhoenix.PubSub.subscribe(pubsub, \"registration_counter\")\n\n# In your process's handle_info:\ndef handle_info(%Signal{} = signal, state) do\n  # Process the signal\n  Logger.info(\"Received signal: #{signal.type} with data: #{inspect(signal.data)}\")\n  {:noreply, state}\nend\n```","ref":"sensors.html#subscribing-to-signals"},{"type":"extras","title":"Testing Sensors and Signals - Sensors","doc":"For a complete example of how to test Sensors and Signals, refer to the registration counter test suite in `test/jido/sensor/examples/user_registration_sensor_test.exs`. This comprehensive test suite demonstrates:\n\n1. Setting up test PubSub environments\n2. Testing event counting and metric calculations\n3. Verifying signal emissions and formats\n4. Handling timing-dependent behavior\n\nThe test patterns shown there can be adapted for testing your own Sensors.","ref":"sensors.html#testing-sensors-and-signals"},{"type":"extras","title":"Best Practices - Sensors","doc":"When working with Signals and Sensors, keep these principles in mind:\n\n1. **Signal Design**\n   - Use clear, consistent signal types\n   - Include enough context in payloads\n   - Follow the CloudEvents spec\n   - Keep payloads focused and minimal\n\n2. **Sensor Implementation**\n   - One responsibility per Sensor\n   - Clear configuration through schema\n   - Efficient state management\n   - Graceful error handling\n\n3. **Testing**\n   - Test both success and failure paths\n   - Verify metric calculations\n   - Check signal formats\n   - Test timing-dependent behavior\n\n4. **Performance**\n   - Use appropriate emission intervals\n   - Batch updates when possible\n   - Monitor memory usage\n   - Clean up old data","ref":"sensors.html#best-practices"},{"type":"extras","title":"Integration with Agents - Sensors","doc":"Signals and Sensors lay the groundwork for advanced agent capabilities by providing:\n\n1. **Real-time Metrics**: Agents can use sensor data to make informed decisions\n2. **Event Streams**: Agents can react to system events through signals\n3. **Status Updates**: Agents can monitor each other's health\n4. **Command Results**: Agents can track the results of their actions\n\nIn the next guide, we'll explore how Server Agents use these signals for coordination and decision-making.","ref":"sensors.html#integration-with-agents"},{"type":"extras","title":"Next Steps - Sensors","doc":"Now that you understand Signals and Sensors, you can explore:\n- Custom signal types\n- Complex metric calculations\n- Event stream processing\n- Multi-sensor coordination\n- Integration with monitoring systems\n- Historical analysis\n\nThe test suite provides many examples of these patterns in action.\n\nRemember: Signals and Sensors are your eyes and ears in the system. Design them thoughtfully to give your agents the information they need to make smart decisions.","ref":"sensors.html#next-steps"},{"type":"extras","title":"Agent Directives","doc":"# Agent Directives\n\nIn our previous guides, we explored how Agents provide stateful wrappers around Actions and how Signals enable real-time monitoring. Now, let's discover how Directives enable agents to modify their own behavior and capabilities at runtime.","ref":"directives.html"},{"type":"extras","title":"Understanding Directives - Agent Directives","doc":"Directives are special instructions that allow agents to modify their own state and capabilities. Think of them as meta-actions that let agents:\n\n1. Queue up new actions to execute (EnqueueDirective)\n2. Learn new capabilities (RegisterActionDirective) \n3. Remove capabilities (DeregisterActionDirective)\n\nThis self-modification ability is crucial for building adaptive agents that can:\n- Respond to new situations by queueing appropriate actions\n- Learn new behaviors by registering actions\n- Optimize themselves by removing unused capabilities\n- Build complex workflows dynamically\n\nLet's see how to implement these patterns.","ref":"directives.html#understanding-directives"},{"type":"extras","title":"Creating Your First Directive-Based Agent - Agent Directives","doc":"We'll create an agent that can adapt its behavior by learning new arithmetic operations. This will demonstrate how directives enable runtime evolution of agent capabilities.\n\n```elixir\ndefmodule MyApp.AdaptiveCalculator do\n  use Jido.Agent,\n    name: \"adaptive_calculator\",\n    description: \"A calculator that can learn new operations\",\n    actions: [\n      # Core actions for self-modification\n      Jido.Actions.Directives.EnqueueAction,\n      Jido.Actions.Directives.RegisterAction,\n      Jido.Actions.Directives.DeregisterAction,\n      \n      # Initial arithmetic capabilities\n      MyApp.Actions.Add,\n      MyApp.Actions.Subtract\n    ],\n    schema: [\n      value: [type: :float, default: 0.0],\n      operations_used: [type: {:list, :atom}, default: []],\n      last_operation: [type: {:or, [:atom, nil]}, default: nil]\n    ]\n\n  # Optional callbacks for tracking operations\n  def on_after_run(agent, result) do\n    case result do\n      %{status: :ok, action: action} ->\n        operations = [action | agent.state.operations_used] |> Enum.uniq()\n        {:ok, %{agent | state: Map.put(agent.state, :operations_used, operations)}}\n      _ ->\n        {:ok, agent}\n    end\n  end\nend\n```\n\nNow let's implement some basic arithmetic actions that can work with our calculator:\n\n```elixir\ndefmodule MyApp.Actions.Add do\n  use Jido.Action,\n    name: \"add\",\n    description: \"Adds a number to the current value\",\n    schema: [\n      value: [type: :float, required: true],\n      amount: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, amount: amount}, _context) do\n    {:ok, %{value: current + amount}}\n  end\nend\n\ndefmodule MyApp.Actions.Multiply do\n  use Jido.Action,\n    name: \"multiply\",\n    description: \"Multiplies the current value by an amount\",\n    schema: [\n      value: [type: :float, required: true],\n      amount: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, amount: amount}, _context) do\n    {:ok, %{value: current * amount}}\n  end\nend\n\ndefmodule MyApp.Actions.Power do\n  use Jido.Action,\n    name: \"power\",\n    description: \"Raises the current value to a power\",\n    schema: [\n      value: [type: :float, required: true],\n      exponent: [type: :float, required: true]\n    ]\n\n  def run(%{value: current, exponent: exp}, _context) do\n    {:ok, %{value: :math.pow(current, exp)}}\n  end\nend\n```","ref":"directives.html#creating-your-first-directive-based-agent"},{"type":"extras","title":"Working with Directives - Agent Directives","doc":"Let's explore how our calculator can use directives to evolve its capabilities:","ref":"directives.html#working-with-directives"},{"type":"extras","title":"1. Queueing New Operations (EnqueueDirective) - Agent Directives","doc":"The EnqueueDirective lets an agent add new instructions to its pending queue. This is useful for building dynamic workflows:\n\n```elixir\n# Create our calculator\ncalculator = MyApp.AdaptiveCalculator.new()\n\n# Start with a simple addition\n{:ok, calculator} = MyApp.AdaptiveCalculator.set(calculator, %{value: 5})\n\n# Queue up a sequence of operations using EnqueueAction\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  [\n    # Utilize a directive to queue up an instruction rather than planning it directly\n    {Jido.Actions.Directives.EnqueueAction, %{\n      action: MyApp.Actions.Add,\n      params: %{amount: 10}\n    }},\n    {Jido.Actions.Directives.EnqueueAction, %{\n      action: MyApp.Actions.Add,\n      params: %{amount: 20}\n    }}\n  ]\n)\n\n# Final value should be 35 (5 + 10 + 20)\ncalculator.state.value #=> 35.0\n```","ref":"directives.html#1-queueing-new-operations-enqueuedirective"},{"type":"extras","title":"2. Learning New Operations (RegisterActionDirective) - Agent Directives","doc":"The RegisterActionDirective lets an agent learn new capabilities at runtime:\n\n```elixir\n# Our calculator doesn't know how to multiply yet\n{:error, _} = MyApp.AdaptiveCalculator.plan(calculator, MyApp.Actions.Multiply)\n\n# Teach it multiplication\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {Jido.Actions.Directives.RegisterAction, %{\n    action_module: MyApp.Actions.Multiply\n  }}\n)\n\n# Now we can multiply!\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {MyApp.Actions.Multiply, %{amount: 2}}\n)\n\ncalculator.state.value #=> 70.0  # (35 * 2)\n```","ref":"directives.html#2-learning-new-operations-registeractiondirective"},{"type":"extras","title":"3. Removing Operations (DeregisterActionDirective) - Agent Directives","doc":"The DeregisterActionDirective lets an agent remove capabilities it no longer needs:\n\n```elixir\n# Remove multiplication if we don't need it anymore\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  {Jido.Actions.Directives.DeregisterAction, %{\n    action_module: MyApp.Actions.Multiply\n  }}\n)\n\n# Trying to multiply now will fail\n{:error, _} = MyApp.AdaptiveCalculator.plan(calculator, MyApp.Actions.Multiply)\n```","ref":"directives.html#3-removing-operations-deregisteractiondirective"},{"type":"extras","title":"Building Complex Self-Modifying Workflows - Agent Directives","doc":"Directives become really powerful when combined into workflows that let agents adapt their behavior based on conditions. Here's an example of a calculator that learns more advanced operations when needed:\n\n```elixir\ndefmodule MyApp.Actions.LearnAdvancedMath do\n  use Jido.Action,\n    name: \"learn_advanced_math\",\n    description: \"Teaches the calculator advanced operations\",\n    schema: [\n      value: [type: :float, required: true]\n    ]\n\n  def run(%{value: value}, _context) do\n    # First register the power operation\n    power_directive = %Jido.Agent.Directive.RegisterActionDirective{\n      action_module: MyApp.Actions.Power\n    }\n\n    # Then queue up a calculation using it\n    calculate_directive = %Jido.Agent.Directive.EnqueueDirective{\n      action: MyApp.Actions.Power,\n      params: %{value: value, exponent: 2}\n    }\n\n    # Return both directives to be applied in order\n    {:ok, [power_directive, calculate_directive]}\n  end\nend\n\n# Use our advanced learning action\ncalculator = MyApp.AdaptiveCalculator.new()\n{:ok, calculator} = MyApp.AdaptiveCalculator.set(calculator, %{value: 5})\n\n# Learn and apply advanced math\n{:ok, calculator} = MyApp.AdaptiveCalculator.cmd(\n  calculator,\n  [\n    {MyApp.Actions.LearnAdvancedMath, %{}},  # This will register Power and queue its use\n    {Jido.Actions.Directives.EnqueueAction, %{  # Then we'll queue another operation\n      action: MyApp.Actions.Add,\n      params: %{amount: 10}\n    }}\n  ]\n)\n\n# Final value: 5^2 + 10 = 35\ncalculator.state.value #=> 35.0\n```","ref":"directives.html#building-complex-self-modifying-workflows"},{"type":"extras","title":"Best Practices - Agent Directives","doc":"When working with directives, keep these principles in mind:\n\n1. **Capability Management**\n   - Only register actions the agent actually needs\n   - Consider deregistering unused actions to keep the agent focused\n   - Track which operations are most frequently used\n\n2. **Directive Chains**\n   - Order directives carefully - registration must happen before usage\n   - Consider using composite actions for complex directive sequences\n   - Validate directive success before proceeding\n\n3. **State Evolution**\n   - Keep track of how agent capabilities change over time\n   - Consider implementing rollback mechanisms for failed directive chains\n   - Use callbacks to monitor and log capability changes\n\n4. **Testing**\n   - Test both successful and failed directive applications\n   - Verify capability addition and removal\n   - Test complex directive chains\n   - Check state consistency after directive application","ref":"directives.html#best-practices"},{"type":"extras","title":"Next Steps - Agent Directives","doc":"Now that you understand directives, you can explore:\n- More complex self-modification patterns\n- Conditional capability loading\n- Dynamic workflow construction\n- State-based capability management\n- Multi-agent capability sharing\n\nRemember: Directives give agents the power to evolve and adapt. Use them thoughtfully to create agents that can grow and optimize themselves while maintaining stability and predictability.","ref":"directives.html#next-steps"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}